import { DbProvider } from './db-provider';
import { Sqlifier } from './sqlifier';
import { RowObject, WhereObject, Condition, XRowset, Sort, SortObject } from '../sql';
import { Executor } from './executor';
import { Command, QueryResult } from './types';
/**
 * 事务隔离级别
 */
export declare enum ISOLATION_LEVEL {
    READ_COMMIT = "READ_COMMIT",
    READ_UNCOMMIT = "READ_UNCOMMIT",
    REPEATABLE_READ = "REPEATABLE_READ",
    SERIALIZABLE = "SERIALIZABLE",
    SNAPSHOT = "SNAPSHOT"
}
export declare class AbortError extends Error {
    code: string;
}
export interface SelectOptions<T extends RowObject = any> {
    where?: WhereObject<T> | Condition | ((table: Readonly<XRowset<T>>) => Condition);
    top?: number;
    offset?: number;
    limit?: number;
    distinct?: boolean;
    sorts?: Sort[] | SortObject<T> | ((rowset: XRowset<T>) => Sort[]);
}
export declare const DefaultConnectionOptions: Partial<ConnectOptions>;
declare abstract class ConnectionClass extends Executor {
    provider: DbProvider;
    constructor(provider: DbProvider, options: ConnectOptions);
    readonly options: ConnectOptions;
    get sqlifier(): Sqlifier;
    protected internalQuery(...args: any): Promise<QueryResult<any, any, any>>;
    /**
     * 是否活动的
     */
    activated: boolean;
    active(): void;
    on: ((event: 'open', listener: () => void) => this) & ((event: 'opened', listener: () => void) => this) & ((event: 'close', listener: () => void) => this) & ((event: 'closed', listener: () => void) => this) & ((event: 'command', listener: (cmd: Command) => void) => this) & ((event: 'error', listener: (error: Error) => any) => this) & ((event: 'begin', listener: (connection: this, id: number) => void) => this) & ((event: 'commit', listener: (connection: this, id: number) => void) => this) & ((event: 'rollback', listener: (connection: this, id: number) => void) => this);
    off: ((event: 'open', listener: () => void) => this) & ((event: 'opened', listener: () => void) => this) & ((event: 'close', listener: () => void) => this) & ((event: 'closed', listener: () => void) => this) & ((event: 'command', listener: (cmd: Command) => void) => this) & ((event: 'error', listener: (error: Error) => any) => this) & ((event: 'begin', listener: (connection: this, id: number) => void) => this) & ((event: 'commit', listener: (connection: this, id: number) => void) => this) & ((event: 'rollback', listener: (connection: this, id: number) => void) => this);
    /**
     * 开启一个事务并在代码执行完成后自动提交，遇到错误时会自动回滚
     * 用户亦可主动调用cancel来取消当前事务，并且产生一个异常中断后续代码执行
     */
    trans<T>(handler: (abort: (message?: string) => void) => Promise<T>, isolationLevel?: ISOLATION_LEVEL): Promise<T>;
    protected abstract doBeginTrans(isolationLevel: ISOLATION_LEVEL): Promise<void>;
    _transCount: number;
    get inTransaction(): boolean;
    /**
     * 当前事务嵌套层数
     */
    get transCount(): number;
    /**
     * 开启一个事务，
     * 支持事务嵌套
     */
    beginTrans(isolationLevel?: ISOLATION_LEVEL): Promise<void>;
    protected abstract createSavePoint(id: number): Promise<void>;
    protected abstract rollbackSavePoint(id: number): Promise<void>;
    protected abstract doCommit(): Promise<void>;
    /**
     * 提交当前事务,如使用嵌套事务，则提交最近事务
     */
    commit(): Promise<void>;
    protected abstract doRollback(): Promise<void>;
    rollback(): Promise<void>;
    private _opened;
    get opened(): boolean;
    protected abstract doOpen(): Promise<void>;
    open(): Promise<void>;
    protected abstract doClose(): Promise<void>;
    /**
     * 重置连接，用于连接池
     */
    protected abstract reset(): Promise<void>;
    /**
     * 等待所有请求完成后关闭连接
     */
    close(): Promise<void>;
    protected abstract doShutdown(): void;
    /**
     * 强制关闭所有连接（慎用）
     */
    shutdown(): void;
    /**
     * 获取当前默认架构
     * @returns
     */
    getSchemaName(): Promise<string>;
    /**
     * 获取当前数据库
     */
    getDatabaseName(): Promise<string>;
    /**
     * 变更所在数据库
     * 当为null时表示登录用户默认数据库
     */
    changeDatabase(name: string): Promise<void>;
    /**
     * 切换到系统数据库，便于创建、删除其它数据操作。
     */
    abstract changeToSysdb(): Promise<void>;
    protected mutex<T>(action: () => Promise<T>): Promise<T>;
    private _inMutex;
}
export declare type ConnectionConstructor = typeof ConnectionClass;
export declare const Connection: ConnectionConstructor;
export interface Connection extends ConnectionClass {
}
export declare type ConnectOptions = {
    /**
     * 数据库方言(必须是已注册的)，与driver二选一，必须安装相应的驱动才可正常使用
     */
    dialect?: string;
    /**
     * 驱动程序，与dialect二选一，优先使用driver
     */
    provider?: DbProvider;
    /**
     * 主机名
     */
    host: string;
    /**
     * 端口号
     */
    port?: number;
    /**
     * 连接用户名
     */
    user: string;
    /**
     * 密码
     */
    password: string;
    /**
     * 数据库名称
     */
    database?: string;
    /**
     * 连接超时时长，单位: ms，默认为15000ms
     */
    connectionTimeout?: number;
    /**
     * 单个查询超时时长,单位: ms，默认为15000ms
     */
    requestTimeout?: number;
};
export declare const INSERT_MAXIMUM_ROWS = 1000;
export {};
