"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.INSERT_MAXIMUM_ROWS = exports.Connection = exports.DefaultConnectionOptions = exports.AbortError = exports.ISOLATION_LEVEL = void 0;
const sql_builder_1 = require("../sql-builder");
const executor_1 = require("./executor");
/**
 * 事务隔离级别
 */
var ISOLATION_LEVEL;
(function (ISOLATION_LEVEL) {
    ISOLATION_LEVEL["READ_COMMIT"] = "READ_COMMIT";
    ISOLATION_LEVEL["READ_UNCOMMIT"] = "READ_UNCOMMIT";
    ISOLATION_LEVEL["REPEATABLE_READ"] = "REPEATABLE_READ";
    ISOLATION_LEVEL["SERIALIZABLE"] = "SERIALIZABLE";
    ISOLATION_LEVEL["SNAPSHOT"] = "SNAPSHOT";
})(ISOLATION_LEVEL = exports.ISOLATION_LEVEL || (exports.ISOLATION_LEVEL = {}));
class AbortError extends Error {
    constructor() {
        super(...arguments);
        this.code = 'ABORT';
    }
}
exports.AbortError = AbortError;
// export interface QueryHandler {
//   (sql: string, params?: Parameter<Scalar, string>[]): Promise<
//     QueryResult<any, any, any>
//   >;
// }
exports.DefaultConnectionOptions = {
    host: 'localhost',
    requestTimeout: 15000,
    connectionTimeout: 30000,
};
class ConnectionClass extends executor_1.Executor {
    constructor(provider, options) {
        super();
        this.provider = provider;
        /**
         * 是否活动的
         */
        this.activated = true;
        this._transCount = 0;
        this._opened = false;
        this._inMutex = false;
        this.options = Object.assign({}, exports.DefaultConnectionOptions, options);
    }
    get sqlifier() {
        return this.provider.sqlifier;
    }
    internalQuery(...args) {
        if (!this.opened) {
            throw new Error(`Connection is not opened yet.`);
        }
        return super.internalQuery(...args);
    }
    active() {
        this.activated = true;
    }
    /**
     * 开启一个事务并在代码执行完成后自动提交，遇到错误时会自动回滚
     * 用户亦可主动调用cancel来取消当前事务，并且产生一个异常中断后续代码执行
     */
    async trans(handler, isolationLevel = ISOLATION_LEVEL.READ_COMMIT) {
        try {
            await this.beginTrans(isolationLevel);
            const res = await handler((message) => {
                throw new AbortError(message || 'Abort.');
            });
            await this.commit();
            return res;
        }
        catch (ex) {
            if (this.inTransaction) {
                await this.rollback();
            }
            throw ex;
        }
    }
    get inTransaction() {
        return this.transCount > 0;
    }
    /**
     * 当前事务嵌套层数
     */
    get transCount() {
        return this._transCount;
    }
    /**
     * 开启一个事务，
     * 支持事务嵌套
     */
    async beginTrans(isolationLevel = ISOLATION_LEVEL.READ_COMMIT) {
        // return this.progress(async () => {
        const id = this._transCount;
        if (this._transCount > 0) {
            await this.createSavePoint(id);
        }
        else {
            await this.doBeginTrans(isolationLevel);
        }
        this._transCount++;
        this.emit('begin', id);
        // });
    }
    /**
     * 提交当前事务,如使用嵌套事务，则提交最近事务
     */
    async commit() {
        if (!this.inTransaction) {
            throw Error(`Connection is not in transaction.`);
        }
        if (this._transCount === 1) {
            await this.doCommit();
        }
        this._transCount--;
        this.emit('commit', this._transCount);
    }
    async rollback() {
        if (!this.inTransaction) {
            throw Error(`Connection is not in transaction.`);
        }
        const id = this._transCount - 1;
        if (id >= 1) {
            await this.rollbackSavePoint(id);
        }
        else {
            await this.doRollback();
        }
        this._transCount--;
        this.emit('rollback', id);
    }
    get opened() {
        return this._opened;
    }
    open() {
        if (this.opened) {
            throw new Error(`Connection is opened.`);
        }
        this.emit('open');
        return this.mutex(async () => {
            await this.doOpen();
            this._opened = true;
            this.emit('opened');
        });
    }
    /**
     * 等待所有请求完成后关闭连接
     */
    close() {
        // if (this.inTransaction) {
        //   throw new Error(
        //     `Connection is in transaction, can't close it before commit or rollback.`
        //   );
        // }
        // if (this._inMutex) {
        //   throw new Error(`Connection is in progress.`);
        // }
        this.emit('close');
        return this.mutex(async () => {
            await this.doClose();
            this._opened = false;
            this.emit('closed');
        });
    }
    /**
     * 强制关闭所有连接（慎用）
     */
    shutdown() {
        this.doShutdown();
        this._opened = false;
        this._transCount = 0;
    }
    /**
     * 获取当前默认架构
     * @returns
     */
    async getSchemaName() {
        return (await this.queryScalar(sql_builder_1.SQL.select(sql_builder_1.SQL.defaultSchema())));
    }
    /**
     * 获取当前数据库
     */
    async getDatabaseName() {
        if (!this.opened) {
            if (this.options.database) {
                return this.options.database;
            }
        }
        return (await this.queryScalar(sql_builder_1.SQL.select(sql_builder_1.SQL.currentDatabase())));
    }
    /**
     * 变更所在数据库
     * 当为null时表示登录用户默认数据库
     */
    async changeDatabase(name) {
        if (name === this.options.database)
            return;
        this.options.database = name;
        if (this.opened) {
            await this.close();
            await this.open();
        }
    }
    async mutex(action) {
        if (this._inMutex) {
            throw new Error(`Connection is in progress.`);
        }
        this._inMutex = true;
        try {
            return action();
        }
        catch (ex) {
            this.emit('error', ex);
            throw ex;
        }
        finally {
            this._inMutex = false;
        }
    }
}
exports.Connection = ConnectionClass;
exports.INSERT_MAXIMUM_ROWS = 1000;
