/**
 * 数据库请求代理，连接和连接池管理的中间件
 */
/// <reference types="node" />
import EventEmitter from 'events';
import { AsScalarType, Assignment, ColumnsOf, Condition, DefaultRowObject, ExecutableStatement, Execute, Expression, Field, InputObject, Parameter, Procedure, RowObject, RowObjectFrom, Scalar, Select, WhereObject, XExpression, XObjectName, XRowset, XTable, XTables, XTableVariant } from '../sql';
import { SelectOptions } from './connection';
import { Sqlifier } from './sqlifier';
import { Command, QueryResult } from './types';
export declare abstract class Executor {
    protected abstract doQuery(sql: string, params?: Parameter[]): Promise<QueryResult<any, any, any>>;
    protected abstract sqlifier: Sqlifier;
    protected _emitter: EventEmitter;
    on(event: 'open', listener: () => void): this;
    on(event: 'opened', listener: () => void): this;
    on(event: 'close', listener: () => void): this;
    on(event: 'closed', listener: () => void): this;
    on(event: 'command', listener: (cmd: Command) => void): this;
    on(event: 'error', listener: (error: Error) => any): this;
    off(event: 'open', listener: () => void): this;
    off(event: 'opened', listener: () => void): this;
    off(event: 'close', listener: () => void): this;
    off(event: 'closed', listener: () => void): this;
    off(event: 'command', listener?: (cmd: Command) => void): this;
    off(event: 'error', listener?: (error: Error) => any): this;
    protected emit(event: string, ...args: any[]): boolean;
    protected internalQuery(...args: any[]): Promise<QueryResult<any, any, any>>;
    query<T extends RowObject = any>(command: Command): Promise<QueryResult<T>>;
    query<T extends RowObject = any>(sql: Select<T>): Promise<QueryResult<T>>;
    /**
     * 执行一个存储过程执行代码
     */
    query<R extends Scalar = number, T extends RowObject = never, O extends [T, ...RowObject[]] = [T]>(sql: Execute<R, O>): Promise<QueryResult<O[0], R, O>>;
    query<T extends RowObject = any>(sql: string): Promise<QueryResult<T>>;
    query<T extends RowObject = any>(sql: string, params?: Parameter[] | Record<string, Scalar>): Promise<QueryResult<T>>;
    query<T extends RowObject = any>(sql: ExecutableStatement | ExecutableStatement[] | string): Promise<QueryResult<T>>;
    query<T extends RowObject = any>(sql: TemplateStringsArray, ...params: any[]): Promise<QueryResult<T>>;
    /**
     * 执行一个查询并获取返回的第一个标量值
     * @param sql
     */
    queryScalar<T extends Scalar = any>(sql: TemplateStringsArray, ...params: Scalar[]): Promise<T | null>;
    queryScalar<T extends Scalar = any>(sql: string, params?: Parameter[]): Promise<T | null>;
    queryScalar<T extends Scalar = any>(sql: string, params?: InputObject): Promise<T | null>;
    queryScalar<T extends RowObject>(sql: Select<T>): Promise<AsScalarType<T> | null>;
    queryScalar<T extends Scalar>(sql: Select<any>): Promise<T | null>;
    queryScalar<T extends Scalar>(expression: Expression<T>): Promise<T | null>;
    /**
     * 插入数据
     */
    insert<T extends RowObject = DefaultRowObject>(table: XObjectName | XTable<any>, fields: string[] | Field<Scalar, ColumnsOf<T>>, values: Scalar[][] | Scalar[]): Promise<number>;
    /**
     * 插入数据
     */
    insert<T extends RowObject = DefaultRowObject>(table: XTable<T> | XObjectName, values: InputObject<T>[] | InputObject): Promise<number>;
    find<T extends RowObject = any, G extends InputObject<T> = InputObject<T>>(table: XTable<T> | XObjectName, results: (rowset: Readonly<XRowset<T>>) => G, where: Condition | WhereObject<T> | ((table: XRowset<T>) => Condition)): Promise<RowObjectFrom<G> | null>;
    find<T extends RowObject = any>(table: XTable<T> | XObjectName, where: Condition | WhereObject<T> | ((table: XRowset<T>) => Condition)): Promise<T | null>;
    /**
     * 简化版的SELECT查询，用于快速查询，如果要用复杂的查询，请使用select语句
     * @param table
     * @param where
     * @param options
     */
    select<T extends RowObject = any, G extends InputObject<T> = InputObject<T>>(table: XObjectName | XRowset<T>, results: (rowset: Readonly<XRowset<T>>) => G, options?: SelectOptions<T>): Promise<RowObjectFrom<G>[]>;
    select<T extends RowObject = any>(table: XObjectName | XRowset<T>, results: ColumnsOf<T>[] | ((rowset: Readonly<XRowset<T>>) => Field[]), options?: SelectOptions<T>): Promise<Partial<T>[]>;
    select<T extends RowObject = any>(table: XObjectName | XRowset<T>, options?: SelectOptions<T>): Promise<T[]>;
    /**
     * 更新表
     */
    update<T extends RowObject = any>(table: XTables<T>, sets: InputObject<T> | Assignment[], where?: WhereObject<T> | Condition | ((table: Readonly<XRowset<T>>) => Condition)): Promise<number>;
    delete<T extends RowObject = any>(table: XTables<T>, where?: WhereObject<T> | Condition | ((table: Readonly<XTable<T>> | Readonly<XTableVariant<T>>) => Condition)): Promise<number>;
    execute<R extends Scalar = number, T extends RowObject = never, O extends [T, ...RowObject[]] = [T]>(spName: XObjectName | Procedure<R, O>, params?: XExpression[]): Promise<QueryResult<T, R, O>>;
}
