"use strict";
/**
 * 数据库请求代理，连接和连接池管理的中间件
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Executor = void 0;
const events_1 = __importDefault(require("events"));
const sql_1 = require("../sql");
const sql_builder_1 = require("../sql-builder");
const connection_1 = require("./connection");
class Executor {
    constructor() {
        this._emitter = new events_1.default();
    }
    on(event, listener) {
        this._emitter.on(event, listener);
        return this;
    }
    off(event, listener) {
        if (!listener) {
            this._emitter.removeAllListeners(event);
        }
        else {
            this._emitter.off(event, listener);
        }
        return this;
    }
    emit(event, ...args) {
        return this._emitter.emit(event, ...args);
    }
    async internalQuery(...args) {
        let command;
        // 如果是AST直接编译
        if (sql_1.Statement.isStatement(args[0]) ||
            sql_1.Document.isDocument(args[0]) ||
            (Array.isArray(args[0]) && sql_1.Statement.isStatement(args[0][0]))) {
            command = this.sqlifier.sqlify(args[0]);
        }
        // 如果是模板字符串
        else if (Array.isArray(args[0] && args[0].raw)) {
            command = {
                sql: this.sqlifier.sqlifyTemplateString(args[0], ...args.slice(1)),
            };
            // 如果是语句数组
        }
        // else if (Array.isArray(args[0]) && Statement.isStatement(args[0][0])) {
        // }
        // command
        else if (args.length === 1 &&
            typeof args[0] === 'object' &&
            typeof args[0].sql === 'string') {
            command = args[0];
            // 如果查询的是表达式，则自动转换为 select xx;
        }
        // else if (args.length === 1 && Expression.isExpression(args[0])) {
        //   command = this.sqlifier.sqlify(SQL.select(args[0]));
        // }
        else {
            if (!(args.length <= 2 && typeof args[0] === 'string')) {
                throw new Error(`Invalid arguments.`);
            }
            let params = [];
            if (typeof args[1] === 'object') {
                params = Object.entries(args[1]).map(([name, value]) => sql_builder_1.SQL.input(name, value));
            }
            command = {
                sql: args[0],
                params,
            };
        }
        try {
            const res = await this.doQuery(command.sql, command.params);
            // 如果有输出参数
            if (command.params && res.output) {
                Object.entries(res.output).forEach(([name, value]) => {
                    const p = command.params.find(p => p.$name === name);
                    p.value = value;
                    if (p.$name === this.sqlifier.options.returnParameterName) {
                        res.returnValue = value;
                    }
                });
            }
            return res;
        }
        finally {
            this.emit('command', command);
        }
    }
    async query(...args) {
        return this.internalQuery(...args);
    }
    async queryScalar(...args) {
        let rows;
        if (args.length === 1 && sql_1.Expression.isExpression(args[0])) {
            rows = (await this.internalQuery(sql_builder_1.SQL.select(args[0]))).rows;
        }
        else {
            rows = (await this.internalQuery(...args)).rows;
        }
        if (rows.length === 0)
            return null;
        return Object.values(rows[0])[0];
    }
    async insert(table, fieldsOrValues, values) {
        let fields;
        let innerValues;
        if (!values) {
            innerValues = (Array.isArray(fieldsOrValues) ? fieldsOrValues : [fieldsOrValues]);
        }
        else {
            fields = fieldsOrValues;
            if (fields.length <= 0) {
                throw new Error(`Fields not allow empty.`);
            }
            // 如果是行数组数组时
            innerValues = sql_1.Insert.isRowDataArray(values, fields.length)
                ? values
                : [values];
        }
        const packedValues = innerValues;
        if (packedValues.length > connection_1.INSERT_MAXIMUM_ROWS) {
            console.warn(`Values exceed ${connection_1.INSERT_MAXIMUM_ROWS} rows, and will be inserted multiple times.`);
        }
        const action = async () => {
            let i = 0;
            let rowsAffected = 0;
            while (i < packedValues.length) {
                const items = packedValues.slice(i, i + connection_1.INSERT_MAXIMUM_ROWS);
                i += connection_1.INSERT_MAXIMUM_ROWS;
                const sql = sql_builder_1.SQL.insert(table, fields).values(items);
                const res = await this.query(sql);
                rowsAffected += res.rowsAffected;
            }
            return rowsAffected;
        };
        // 启用事务
        if (this instanceof connection_1.Connection && !this.inTransaction) {
            return this.trans(action);
        }
        return action();
    }
    async find(table, resultsOrwhere, where) {
        const t = sql_1.Rowset.isRowset(table) ? table : sql_1.Table.create(table);
        let results;
        if (!where) {
            where = resultsOrwhere;
        }
        else {
            results = resultsOrwhere;
        }
        let columns;
        if (results) {
            columns = results(t);
        }
        else {
            columns = t.star;
        }
        const sql = sql_builder_1.SQL.select(columns)
            .top(1)
            .from(t)
            .where(typeof where === 'function' ? where(t) : where);
        const rows = (await this.internalQuery(sql)).rows;
        if (rows.length > 0) {
            return rows[0];
        }
        return null;
    }
    async select(table, arg2, arg3) {
        let options;
        let results;
        if (typeof arg2 === 'function' || Array.isArray(arg2)) {
            results = arg2;
            options = arg3;
        }
        else {
            options = arg2;
        }
        const { where, sorts, offset, limit } = options || {};
        let columns;
        const t = (sql_1.Rowset.isRowset(table) ? table : sql_1.Table.create(table));
        if (!results) {
            columns = t.star;
        }
        else if (typeof results === 'function') {
            columns = results(t);
        }
        else {
            columns = results.map(f => (sql_1.Field.isField(f) ? f : t[f]));
        }
        let sql;
        if (Array.isArray(columns)) {
            sql = sql_builder_1.SQL.select(...columns).from(t);
        }
        else {
            sql = sql_builder_1.SQL.select(columns).from(t);
        }
        if (where) {
            sql.where(typeof where === 'function' ? where(t) : where);
        }
        if (sorts) {
            if (Array.isArray(sorts)) {
                sql.orderBy(...sorts);
            }
            else if (typeof sorts === 'function') {
                sql.orderBy(...sorts(t));
            }
            else {
                sql.orderBy(sorts);
            }
        }
        if (!(typeof offset === 'undefined')) {
            sql.offset(offset);
        }
        if (!(typeof limit === 'undefined')) {
            sql.limit(limit);
        }
        const res = await this.query(sql);
        return res.rows;
    }
    /**
     * 更新表
     */
    async update(table, sets, where) {
        const t = sql_1.Table.isTable(table) || sql_1.TableVariant.isTableVariant(table)
            ? table
            : sql_1.Table.create(table);
        const sql = sql_builder_1.SQL.update(t).set(sets);
        if (where) {
            if (typeof where === 'function') {
                where = where(t);
            }
            sql.where(where);
        }
        const res = await this.query(sql);
        return res.rowsAffected;
    }
    async delete(table, where) {
        const t = sql_1.Table.isTable(table) || sql_1.TableVariant.isTableVariant(table)
            ? table
            : sql_1.Table.create(table);
        const sql = sql_builder_1.SQL.delete(t);
        if (where) {
            if (typeof where === 'function') {
                where = where(t);
            }
            sql.where(where);
        }
        const res = await this.query(sql);
        return res.rowsAffected;
    }
    async execute(spName, params) {
        const sql = sql_builder_1.SQL.execute(spName, params);
        const res = await this.query(sql);
        return res;
    }
}
exports.Executor = Executor;
