"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pool = exports.DefaultPoolOptions = void 0;
const tarn_1 = __importDefault(require("tarn"));
const register_1 = require("../register");
const sql_builder_1 = require("../sql-builder");
const pool_executor_1 = require("./pool-executor");
const events_1 = __importDefault(require("events"));
exports.DefaultPoolOptions = {
    max: 5,
    min: 1,
    connectTimeout: 30000,
    idleTimeout: 30000,
};
class Pool {
    constructor(provider, poolOptions, connectionOptions) {
        this._emitter = new events_1.default();
        this._healthy = undefined;
        if (typeof provider === 'string') {
            this.provider = (0, register_1.getProvider)(provider);
        }
        else {
            this.provider = provider;
        }
        this.poolOptions = Object.assign({
            max: 5,
            min: 1,
            connectTimeout: 30000,
            idleTimeout: 30000,
        }, poolOptions);
        this.connectionOptions = connectionOptions;
    }
    on(event, handler) {
        this._emitter.on(event, handler);
        return this;
    }
    off(event, handler) {
        if (!handler) {
            this._emitter.removeAllListeners(event);
        }
        else {
            this._emitter.off(event, handler);
        }
        return this;
    }
    get opened() {
        return !!this.pool;
    }
    checkOpened() {
        if (!this.opened) {
            throw new Error(`Pool is not opened yet.`);
        }
    }
    /**
     * 当前连接池健康状态
     */
    get healthy() {
        this.checkOpened();
        return this._healthy;
    }
    async test() {
        let conn;
        try {
            conn = await this._create();
            return true;
        }
        catch {
            return false;
        }
        finally {
            if (conn) {
                await conn.close();
            }
        }
    }
    async _create() {
        const conn = this.provider.getConnection(this.connectionOptions);
        try {
            await conn.open();
            this.setHealthy(true);
            this._emitter.emit('connection-open', conn);
        }
        catch (error) {
            if (this.pool.numUsed() + this.pool.numFree() <= 0) {
                this.setHealthy(false);
            }
            throw error;
        }
        return conn;
    }
    setHealthy(healthy) {
        if (this._healthy !== healthy) {
            this._healthy === healthy;
            this._emitter.emit('healthy-changed', this._healthy);
        }
    }
    async open() {
        await this.test();
        this.pool = new tarn_1.default.Pool({
            max: this.poolOptions.max,
            min: this.poolOptions.min,
            createTimeoutMillis: this.poolOptions.connectTimeout,
            log: (msg) => {
                this._emitter.emit('log', msg);
            },
            validate: (async (conn) => {
                try {
                    await conn.query(sql_builder_1.SQL.select(1));
                    return true;
                }
                catch {
                    return false;
                }
            }),
            create: this._create.bind(this),
            destroy: async (conn) => {
                conn.close().finally(() => {
                    this._emitter.emit('connection-close', conn);
                });
            },
            propagateCreateError: true,
        });
    }
    async close() {
        this.checkOpened();
        await this.pool?.destroy();
        this.pool = undefined;
        this._healthy = undefined;
    }
    /**
     * 获取连接
     */
    acquire() {
        this.checkOpened();
        return this.pool.acquire().promise.then(conn => {
            this._emitter.emit('acquire', conn);
            return conn;
        });
    }
    release(conn) {
        this.checkOpened();
        this.pool.release(conn);
        this._emitter.emit('release', conn);
    }
    /**
     * 全局执行器，该执行器在执行SQL时会自动请求和释放连接，但不保证所有执行的SQL都在同一个数据库连接中。
     */
    get executor() {
        if (!this._executor) {
            this._executor = new pool_executor_1.PoolExecutor(this);
        }
        return this._executor;
    }
    /**
     * 当前总连接数
     */
    get size() {
        this.checkOpened();
        return (this.pool.numFree() +
            this.pool.numUsed() +
            this.pool.numPendingCreates());
    }
    /**
     * 当前可用连接数
     */
    get available() {
        this.checkOpened();
        return this.pool.numFree();
    }
    /**
     * 当前等待请求数
     */
    get pending() {
        return this.pool.numPendingAcquires();
    }
    /**
     * 当前使用中连接数
     */
    get borrowed() {
        return this.pool.numUsed();
    }
}
exports.Pool = Pool;
