import { XObjectName, Raw, Field, DbType, Scalar, VirtualExpression, Expression, VirtualStatement, Statement, VirtualCondition, Condition, Parameter, DBObject, Variant, Document, ObjectName, Annotation, Select, Update, Insert, Delete, Declare, Execute, Assignment, CreateTable, AlterTable, DropTable, CreateView, AlterView, DropView, CreateProcedure, AlterProcedure, DropProcedure, CreateFunction, AlterFunction, DropFunction, CreateIndex, DropIndex, CreateSequence, DropSequence, Block, If, While, Break, Continue, CreateDatabase, AlterDatabase, DropDatabase, GroupExpression, ValuedSelect, UnaryOperation, BinaryOperation, Rowset, NamedSelect, BuiltIn, SelectColumn, Star, WithSelect, With, Union, Case, When, GroupCondition, BinaryLogicCondition, BinaryCompareCondition, UnaryCompareCondition, ExistsCondition, UnaryLogicCondition, ScalarFuncInvoke, TableFuncInvoke, Join, Sort, TableVariant, ProcedureParameter, Operation, FunctionParameter, Return, PARAMETER_DIRECTION, CreateSchema, DropSchema } from '../sql';
import { Command } from './types';
import { VirtualTranslator } from './virtual-translator';
/**
 * 编译选项
 */
export interface SqlOptions {
    /**
     * 是否启用严格模式，默认启用
     * 如果为false，则生成的SQL标识不会被[]或""包括
     */
    strict?: boolean;
    /**
     * 标识符引用，左
     */
    quotedLeft: string;
    /**
     * 标识符引用，右
     */
    quotedRight: string;
    /**
     * 默认架构名称
     * 当对表结构进行操作时，如果未指定架构，则自动使用该架构操作
     */
    defaultSchema: string;
    /**
     * 参数前缀
     */
    parameterPrefix?: string;
    /**
     * 变量前缀
     */
    variantPrefix: string;
    /**
     * 返回参数名称
     */
    returnParameterName?: string;
    /**
     * 集合别名连接词，默认为 'AS'
     */
    setsAliasJoinWith?: string;
    /**
     * 输出参数尾词，默认为 'OUT'
     */
    parameterOutWord?: string;
    /**
     * 字段别名连接字符器，默认为 'AS'
     */
    fieldAliasJoinWith?: string;
    /**
     * 语句结尾
     */
    statementEnd?: string;
}
/**
 * AST到SQL的编译器基类，包含大部分标准实现
 */
export declare abstract class Sqlifier {
    constructor(options?: SqlOptions);
    readonly options: SqlOptions;
    /**
     * 标准SQL转换器
     */
    abstract readonly virtualTranslator: VirtualTranslator;
    sqlifyObjectName(name: XObjectName, builtIn?: boolean): string;
    /**
     * 通过模板字符串创建一个SQL命令
     */
    sqlifyTemplateString(arr: TemplateStringsArray, ...paramValues: any[]): string;
    protected sqlifyField(field: Field | Raw): string;
    abstract sqlifyType(type: DbType | Raw): string;
    /**
     * 将原始SQL类型转换为DbType
     * @param type
     */
    abstract parseRawType(type: string): DbType;
    /**
     *
     * @param operation 将标准操作编译成AST
     */
    protected translateVirtualSql<T extends Scalar>(operation: VirtualExpression<T>): Expression<T>;
    protected translateVirtualSql<T extends Scalar>(operation: VirtualStatement): Statement;
    protected translateVirtualSql<T extends Scalar>(operation: VirtualCondition): Condition;
    /**
     * 编译Insert语句中的字段，取掉表别名
     * @param field 字段
     */
    protected sqlifyInsertField(field: Field<Scalar, string>): string;
    /**
     * 标识符转换，避免关键字被冲突问题
     * @param {string} name 标识符
     */
    quoted(name: string): string;
    /**
     * 向参数列表中添加参数并返回当前参数的参数名
     * @param {array} values 参数列表
     * @param {any} value 参数值
     */
    protected sqlifyParameter(param: Parameter<Scalar, string>, params: Set<Parameter<Scalar, string>>): string;
    protected sqlifyParameterName(p: Parameter<Scalar, string>): string;
    protected sqlifyVariantName(name: string): string;
    protected sqlifyTableVariantName(name: string): string;
    protected sqlifyIdentifier(identifier: DBObject<string>): string;
    sqlifyVariant(variant: Variant): string;
    /**
     * 编译字面量
     */
    abstract sqlifyLiteral(literal: Scalar | Raw, type?: DbType): string;
    /**
     * 将AST编译成一个可供执行的命令
     */
    sqlify(ast: Statement[] | Statement | Document | Raw): Command;
    abstract parseObjectName(name: XObjectName): ObjectName;
    protected sqlifyStatement(
    /**
     * AST
     */
    statement: Statement | Raw, 
    /**
     * 参数容器
     */
    params?: Set<Parameter<Scalar, string>>
    /**
     * 父级AST
     */
    ): string;
    sqlifyReturn(statement: Return, params: Set<Parameter<Scalar, string>> | undefined): string;
    private _invalidAST;
    abstract sqlifyContinue(statement: Continue): string;
    abstract sqlifyBreak(statement: Break): string;
    abstract sqlifyWhile(statement: While, params?: Set<Parameter<Scalar, string>>): string;
    abstract sqlifyIf(statement: If, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyStatements(statements: Statement[], 
    /**
     * 参数容器
     */
    params?: Set<Parameter<Scalar, string>>): string;
    protected abstract sqlifyAnnotation(statement: Annotation): string;
    protected abstract sqlifyDropSequence(statement: DropSequence): string;
    protected abstract sqlifyCreateSequence(statement: CreateSequence): string;
    protected abstract sqlifyBlock(statement: Block, 
    /**
     * 参数容器
     */ params?: Set<Parameter<Scalar, string>>): string;
    protected abstract sqlifyDropIndex(statement: DropIndex): string;
    protected abstract sqlifyCreateIndex(statement: CreateIndex): string;
    protected abstract sqlifyDropFunction(statement: DropFunction): string;
    protected abstract sqlifyAlterFunction(statement: AlterFunction): string;
    protected abstract sqlifyCreateFunction(statement: CreateFunction): string;
    protected abstract sqlifyDropProcedure(statement: DropProcedure): string;
    protected abstract sqlifyAlterProcedure(statement: AlterProcedure): string;
    protected abstract sqlifyCreateProcedure(statement: CreateProcedure): string;
    protected sqlifyAlterView(statement: AlterView<any, string>): string;
    protected abstract sqlifyDropView(statement: DropView): string;
    protected sqlifyCreateView(statement: CreateView<any, string>): string;
    protected abstract sqlifyDropTable(statement: DropTable): string;
    protected abstract sqlifyAlterTable(statement: AlterTable): string;
    protected abstract sqlifyCreateTable(statement: CreateTable): string;
    protected abstract sqlifyCreateDatabase(statement: CreateDatabase): string;
    protected abstract sqlifyCreateSchema(statement: CreateSchema): string;
    protected abstract sqlifyDropSchema(statement: DropSchema): string;
    protected abstract sqlifyAlterDatabase(statement: AlterDatabase): string;
    protected abstract sqlifyDropDatabase(statement: DropDatabase): string;
    protected sqlifyGroupExpression(expr: GroupExpression<Scalar>, params?: Set<Parameter<Scalar, string>>): string;
    /**
     * SELECT 语句 当值使用
     */
    protected sqlifyValuedSelect(expr: ValuedSelect<Scalar>, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyOperation(operation: Operation, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyUnaryOperation(opt: UnaryOperation<Scalar>, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyRowsetName(rowset: Rowset | Raw, forField?: boolean): string;
    protected sqlifyNamedSelect(rowset: NamedSelect, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyBuildIn(buildIn: BuiltIn<string>): string;
    protected sqlifyColumn(column: SelectColumn<Scalar, string> | Star | Expression<Scalar> | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyWithSelect(item: WithSelect<any> | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyWith(withs: With | Raw, params?: Set<Parameter<Scalar, string>>): string;
    /**
     * 将多段SQL合成一个字符串
     * @param sqls
     */
    abstract joinBatchSql(...sqls: string[]): string;
    protected sqlifyDocument(doc: Document, params?: Set<Parameter<Scalar, string>>): string;
    protected abstract sqlifyExecute(exec: Execute, params: Set<Parameter<Scalar, string>>): string;
    protected sqlifyInvokeArgumentList(args: Expression<Scalar>[], params?: Set<Parameter<Scalar, string>>): string;
    protected abstract sqlifyDirection(direction: PARAMETER_DIRECTION): string;
    protected sqlifyExecuteArgumentList(args: Expression<Scalar>[], params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyUnion(union: Union, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyCase(caseExpr: Case<any>, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyWhen(when: When<any> | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyGroupCondition(expr: GroupCondition, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyBinaryLogicCondition(expr: BinaryLogicCondition, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyBinaryCompareCondition(expr: BinaryCompareCondition, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyBinaryOperation(expr: BinaryOperation<Scalar>, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyUnaryCompareCondition(expr: UnaryCompareCondition, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyExistsCondition(expr: ExistsCondition, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyUnaryLogicCondition(expr: UnaryLogicCondition, params?: Set<Parameter<Scalar, string>>): string;
    sqlifyExpression(expr: Expression<Scalar> | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyInvokeArgs(arg: Expression | Star | BuiltIn, params?: Set<Parameter>): string;
    protected sqlifyStar(arg: Star<any>): string;
    /**
     * 函数调用
     * @param {*} invoke
     * @param {*} params
     * @returns
     * @memberof Executor
     */
    protected sqlifyScalarFuncInvoke(invoke: ScalarFuncInvoke, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyTableFuncInvoke(invoke: TableFuncInvoke, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyJoin(join: Join | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifySort(sort: Sort | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifySelect(select: Select, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyOffsetLimit(select: Select<any>, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyFrom(table: Rowset | Raw, params?: Set<Parameter<Scalar, string>>): string;
    sqlifyCondition(condition: Condition | Raw, params?: Set<Parameter<Scalar, string>>): string;
    protected assertAst(value: any, message?: string): asserts value;
    protected sqlifyInsert(insert: Insert, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyAssignment(assign: Assignment<Scalar>, params?: Set<Parameter<Scalar, string>>): string;
    protected abstract sqlifyTableVariantDeclare(declare: TableVariant): string;
    protected sqlifyVariantDeclare(varDec: Variant): string;
    protected abstract sqlifyProcedureParameter(varDec: ProcedureParameter): string;
    protected abstract sqlifyFunctionParameter(varDec: FunctionParameter): string;
    protected abstract sqlifyDeclare(declare: Declare): string;
    protected sqlifyUpdate(update: Update, params?: Set<Parameter<Scalar, string>>): string;
    protected sqlifyDelete(del: Delete, params?: Set<Parameter<Scalar, string>>): string;
}
