"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sqlifier = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const assert_1 = __importDefault(require("assert"));
const sql_1 = require("../sql");
const DEFAULT_COMPILE_OPTIONS = {
    strict: true,
    defaultSchema: 'dbo',
    /**
     * 标识符引用，左
     */
    quotedLeft: '"',
    /**
     * 标识符引用，右
     */
    quotedRight: '"',
    /**
     * 参数前缀
     */
    parameterPrefix: '@',
    statementEnd: ';',
    /**
     * 变量前缀
     */
    variantPrefix: '',
    /**
     * 返回参数名称
     */
    returnParameterName: '__RETURN_VALUE__',
};
/**
 * AST到SQL的编译器基类，包含大部分标准实现
 */
class Sqlifier {
    constructor(options) {
        this.options = Object.assign({}, DEFAULT_COMPILE_OPTIONS, options);
    }
    sqlifyObjectName(name, builtIn = false) {
        // TIPS: buildIn 使用小写原因是数据库默认值会被自动转换为小写从而产生结果差异，造成不必要的数据架构变化。
        if (sql_1.Raw.isRaw(name)) {
            return name.$sql;
        }
        if (typeof name === 'string') {
            name = this.parseObjectName(name);
        }
        let str = builtIn ? name.name : this.quoted(name.name);
        if (name.schema) {
            str = this.quoted(name.schema) + '.' + str;
        }
        // if (name.database) {
        //   str = this.quoted(name.database) + '.' + str;
        // }
        return str;
    }
    /**
     * 通过模板字符串创建一个SQL命令
     */
    sqlifyTemplateString(arr, ...paramValues) {
        // const params: Parameter[] = [];
        let sql = arr[0];
        for (let i = 0; i < arr.length - 1; i++) {
            sql += this.sqlifyLiteral(paramValues[i]);
            sql += arr[i + 1];
        }
        return sql;
    }
    sqlifyField(field) {
        if (sql_1.Raw.isRaw(field)) {
            return field.$sql;
        }
        let sql = '';
        if (field.$table) {
            sql = this.sqlifyRowsetName(field.$table, true);
            sql += '.' + this.quoted(field.$name);
            return sql;
        }
        return this.quoted(field.$name);
    }
    translateVirtualSql(operation) {
        const transFn = Reflect.get(this.virtualTranslator, operation.$name);
        return transFn.call(this.virtualTranslator, ...operation.$datas);
    }
    /**
     * 编译Insert语句中的字段，取掉表别名
     * @param field 字段
     */
    sqlifyInsertField(field) {
        if (typeof field.$name === 'string')
            return this.quoted(field.$name);
        return this.quoted(field.$name[0]);
    }
    /**
     * 标识符转换，避免关键字被冲突问题
     * @param {string} name 标识符
     */
    quoted(name) {
        if (this.options.strict) {
            return (this.options.quotedLeft +
                name.replace(this.options.quotedRight, this.options.quotedRight + this.options.quotedRight) +
                this.options.quotedRight);
        }
        return name;
    }
    /**
     * 向参数列表中添加参数并返回当前参数的参数名
     * @param {array} values 参数列表
     * @param {any} value 参数值
     */
    sqlifyParameter(param, params) {
        if (!params) {
            throw new Error(`Params must provide when use parameter`);
        }
        params.add(param);
        return this.sqlifyParameterName(param);
    }
    sqlifyParameterName(p) {
        return this.options.parameterPrefix + (p.$name || '');
    }
    sqlifyVariantName(name) {
        return this.options.variantPrefix + name;
    }
    sqlifyTableVariantName(name) {
        return this.options.variantPrefix + name;
    }
    sqlifyIdentifier(identifier) {
        return this.sqlifyObjectName(identifier.$name, identifier.$builtin);
    }
    // /**
    //  * 通过模板参数创建一个SQL命令
    //  */
    // makeCommand(arr: TemplateStringsArray, ...paramValues: any[]): Command {
    //   const params: Parameter[] = [];
    //   let sql: string = arr[0];
    //   for (let i = 0; i < arr.length - 1; i++) {
    //     const name = "__p__" + i;
    //     const param = SQL.input(name, paramValues[i]);
    //     sql += this.stringifyParameterName(param);
    //     sql += arr[i + 1];
    //     params.push(param);
    //   }
    //   return {
    //     sql,
    //     params,
    //   };
    // }
    sqlifyVariant(variant) {
        return this.sqlifyVariantName(variant.$name);
    }
    /**
     * 将AST编译成一个可供执行的命令
     */
    sqlify(ast) {
        const params = new Set();
        let sql;
        if (sql_1.Raw.isRaw(ast)) {
            return {
                sql: ast.$sql,
            };
        }
        if (sql_1.Document.isDocument(ast)) {
            sql = this.sqlifyDocument(ast, params);
        }
        else if (Array.isArray(ast)) {
            sql = this.sqlifyStatements(ast, params);
        }
        else {
            sql = this.sqlifyStatement(ast, params);
        }
        return {
            sql,
            params: Array.from(params),
        };
    }
    sqlifyStatement(
    /**
     * AST
     */
    statement, 
    /**
     * 参数容器
     */
    params
    /**
     * 父级AST
     */
    ) {
        if (sql_1.Raw.isRaw(statement)) {
            return statement.$sql;
        }
        if (sql_1.Annotation.isAnnotation(statement)) {
            return this.sqlifyAnnotation(statement);
        }
        if (sql_1.Select.isSelect(statement)) {
            return this.sqlifySelect(statement, params);
        }
        else if (sql_1.Update.isUpdate(statement)) {
            return this.sqlifyUpdate(statement, params);
        }
        else if (sql_1.Insert.isInsert(statement)) {
            return this.sqlifyInsert(statement, params);
        }
        else if (sql_1.Delete.isDelete(statement)) {
            return this.sqlifyDelete(statement, params);
        }
        else if (sql_1.Declare.isDeclare(statement)) {
            return this.sqlifyDeclare(statement);
        }
        else if (sql_1.Execute.isExecute(statement)) {
            return this.sqlifyExecute(statement, params);
        }
        else if (sql_1.Assignment.isAssignment(statement)) {
            return this.sqlifyAssignment(statement, params);
        }
        else if (sql_1.CreateTable.isCreateTable(statement)) {
            return this.sqlifyCreateTable(statement);
        }
        else if (sql_1.AlterTable.isAlterTable(statement)) {
            return this.sqlifyAlterTable(statement);
        }
        else if (sql_1.DropTable.isDropTable(statement)) {
            return this.sqlifyDropTable(statement);
        }
        else if (sql_1.CreateView.isCreateView(statement)) {
            return this.sqlifyCreateView(statement);
        }
        else if (sql_1.AlterView.isAlterView(statement)) {
            return this.sqlifyAlterView(statement);
        }
        else if (sql_1.DropView.isDropView(statement)) {
            return this.sqlifyDropView(statement);
        }
        else if (sql_1.CreateProcedure.isCreateProcedure(statement)) {
            return this.sqlifyCreateProcedure(statement);
        }
        else if (sql_1.AlterProcedure.isAlterProcedure(statement)) {
            return this.sqlifyAlterProcedure(statement);
        }
        else if (sql_1.DropProcedure.isDropProcedure(statement)) {
            return this.sqlifyDropProcedure(statement);
        }
        else if (sql_1.CreateFunction.isCreateFunction(statement)) {
            return this.sqlifyCreateFunction(statement);
        }
        else if (sql_1.AlterFunction.isAlterFunction(statement)) {
            return this.sqlifyAlterFunction(statement);
        }
        else if (sql_1.DropFunction.isDropFunction(statement)) {
            return this.sqlifyDropFunction(statement);
        }
        else if (sql_1.CreateIndex.isCreateIndex(statement)) {
            return this.sqlifyCreateIndex(statement);
        }
        else if (sql_1.DropIndex.isDropIndex(statement)) {
            return this.sqlifyDropIndex(statement);
        }
        else if (sql_1.CreateSequence.isCreateSequence(statement)) {
            return this.sqlifyCreateSequence(statement);
        }
        else if (sql_1.DropSequence.isDropSequence(statement)) {
            return this.sqlifyDropSequence(statement);
        }
        else if (sql_1.Block.isBlock(statement)) {
            return this.sqlifyBlock(statement, params);
        }
        else if (sql_1.VirtualStatement.isVirtualStatement(statement)) {
            return this.sqlifyStatement(this.translateVirtualSql(statement), params);
        }
        else if (sql_1.Return.isReturn(statement)) {
            return this.sqlifyReturn(statement, params);
        }
        else if (sql_1.If.isIf(statement)) {
            return this.sqlifyIf(statement, params);
        }
        else if (sql_1.While.isWhile(statement)) {
            return this.sqlifyWhile(statement, params);
        }
        else if (sql_1.Break.isBreak(statement)) {
            return this.sqlifyBreak(statement);
        }
        else if (sql_1.Continue.isContinue(statement)) {
            return this.sqlifyContinue(statement);
        }
        else if (sql_1.CreateDatabase.isCreateDatabase(statement)) {
            return this.sqlifyCreateDatabase(statement);
        }
        else if (sql_1.AlterDatabase.isAlterDatabase(statement)) {
            return this.sqlifyAlterDatabase(statement);
        }
        else if (sql_1.DropDatabase.isDropDatabase(statement)) {
            return this.sqlifyDropDatabase(statement);
        }
        else if (sql_1.CreateSchema.isCreateSchema(statement)) {
            return this.sqlifyCreateSchema(statement);
        }
        else if (sql_1.DropSchema.isDropSchema(statement)) {
            return this.sqlifyDropSchema(statement);
        }
        // else if (Use.isUse(statement)) {
        //   return this.sqlifyUse(statement);
        // }
        throw this._invalidAST('statement', statement);
    }
    sqlifyReturn(statement, params) {
        let sql = 'RETURN';
        if (statement.$value) {
            sql += ' ' + this.sqlifyExpression(statement.$value, params);
        }
        return sql;
    }
    // sqlifyUse(statement: Use): string {
    //   return `USE ${this.sqlifyObjectName(statement.$database)}`;
    // }
    _invalidAST(type, value) {
        console.debug(`Invalid ${type} AST：`, value);
        return new Error(`Invalid ${type} AST.`);
    }
    sqlifyStatements(statements, 
    /**
     * 参数容器
     */
    params) {
        return statements
            .map(statement => this.sqlifyStatement(statement, params))
            .join('\n');
    }
    sqlifyAlterView(statement) {
        this.assertAst(statement.$body, 'AlterView has no body statement.');
        return `ALTER VIEW ${this.sqlifyObjectName(statement.$name)} AS ${this.sqlifySelect(statement.$body)}`;
    }
    sqlifyCreateView(statement) {
        this.assertAst(statement.$body, 'CreateView statement has no statements body.');
        return `CREATE VIEW ${this.sqlifyObjectName(statement.$name)} AS ${this.sqlifySelect(statement.$body)}`;
    }
    sqlifyGroupExpression(expr, params) {
        return `(${this.sqlifyExpression(expr.$inner, params)})`;
    }
    /**
     * SELECT 语句 当值使用
     */
    sqlifyValuedSelect(expr, params) {
        return `(${this.sqlifySelect(expr.$select, params)})`;
    }
    sqlifyOperation(operation, params) {
        if (sql_1.UnaryOperation.isUnaryOperation(operation)) {
            return this.sqlifyUnaryOperation(operation, params);
        }
        if (sql_1.BinaryOperation.isBinaryOperation(operation)) {
            return this.sqlifyBinaryOperation(operation, params);
        }
        throw this._invalidAST('operation', operation);
    }
    sqlifyUnaryOperation(opt, params) {
        return opt.$operator + this.sqlifyExpression(opt.$value, params);
    }
    sqlifyRowsetName(rowset, forField = false) {
        if (sql_1.Raw.isRaw(rowset))
            return rowset.$sql;
        if (rowset.$alias) {
            return this.quoted(rowset.$alias);
        }
        // 如果是表变量
        if (sql_1.TableVariant.isTableVariant(rowset)) {
            const result = this.sqlifyTableVariantName(rowset.$name);
            if (forField) {
                return this.quoted(result);
            }
            return result;
        }
        if (rowset.$name) {
            return this.sqlifyObjectName(rowset.$name);
        }
        throw new Error('Rowset must have alias or name.');
    }
    sqlifyNamedSelect(rowset, params) {
        return ('(' +
            this.sqlifySelect(rowset.$select, params) +
            ') AS ' +
            this.sqlifyRowsetName(rowset));
    }
    // protected sqlifyTableInvoke(): string {
    //   throw new Error('Method not implemented.');
    // }
    sqlifyBuildIn(buildIn) {
        return buildIn.$name;
    }
    sqlifyColumn(column, params) {
        if (sql_1.SelectColumn.isSelectColumn(column)) {
            return `${this.sqlifyExpression(column.$expr, params)} AS ${this.quoted(column.$name)}`;
        }
        if (sql_1.Raw.isRaw(column)) {
            return column.$sql;
        }
        if (sql_1.BuiltIn.isBuiltIn(column)) {
            return this.sqlifyBuildIn(column);
        }
        if (sql_1.Star.isStar(column)) {
            return this.sqlifyStar(column);
        }
        if (sql_1.Literal.isLiteral(column)) {
            // 自动附带常量返回类型
            return this.sqlifyLiteral(column.$value, column.$dbType || sql_1.Literal.parseValueType(column.$value));
        }
        return this.sqlifyExpression(column, params);
    }
    sqlifyWithSelect(item, params) {
        if (sql_1.Raw.isRaw(item))
            return item.$sql;
        return `${this.quoted(item.$name)} AS (${this.sqlifySelect(item.$select, params)})`;
    }
    sqlifyWith(withs, params) {
        if (sql_1.Raw.isRaw(withs))
            return withs.$sql;
        return ('WITH ' +
            withs.$rowsets.map(item => this.sqlifyWithSelect(item, params)).join(', '));
    }
    sqlifyDocument(doc, params) {
        return doc.statements
            .map(statement => this.sqlifyStatement(statement, params))
            .join('\n');
    }
    sqlifyInvokeArgumentList(args, params) {
        return args.map(expr => this.sqlifyExpression(expr, params)).join(', ');
    }
    sqlifyExecuteArgumentList(args, params) {
        return args
            .map(ast => {
            let sql = this.sqlifyExpression(ast, params);
            if (sql_1.Parameter.isParameter(ast)) {
                sql += ' ' + this.sqlifyDirection(ast.direction);
            }
            return sql;
        })
            .join(', ');
    }
    sqlifyUnion(union, params) {
        return ('UNION ' +
            (union.$all ? 'ALL ' : '') +
            (sql_1.Select.isSelect(union.$select)
                ? this.sqlifySelect(union.$select, params)
                : this.sqlifyRowsetName(union.$select)));
    }
    sqlifyCase(caseExpr, params) {
        let fragment = 'CASE';
        if (caseExpr.$expr)
            fragment += ' ' + this.sqlifyExpression(caseExpr.$expr, params);
        fragment +=
            ' ' +
                caseExpr.$whens.map(when => this.sqlifyWhen(when, params)).join(' ');
        if (caseExpr.$default)
            fragment += ' ELSE ' + this.sqlifyExpression(caseExpr.$default, params);
        fragment += ' END';
        return fragment;
    }
    sqlifyWhen(when, params) {
        if (sql_1.Raw.isRaw(when))
            return when.$sql;
        return ('WHEN ' +
            (sql_1.Condition.isCondition(when.$expr)
                ? this.sqlifyCondition(when.$expr, params)
                : this.sqlifyExpression(when.$expr, params)) +
            ' THEN ' +
            this.sqlifyExpression(when.$value, params));
    }
    sqlifyGroupCondition(expr, params) {
        return '(' + this.sqlifyCondition(expr.$inner, params) + ')';
    }
    sqlifyBinaryLogicCondition(expr, params) {
        return (this.sqlifyCondition(expr.$left, params) +
            ' ' +
            expr.$operator +
            ' ' +
            this.sqlifyCondition(expr.$right, params));
    }
    sqlifyBinaryCompareCondition(expr, params) {
        return (this.sqlifyExpression(expr.$left, params) +
            ' ' +
            expr.$operator +
            ' ' +
            (Array.isArray(expr.$right)
                ? '(' + expr.$right.map(p => this.sqlifyExpression(p, params)) + ')'
                : this.sqlifyExpression(expr.$right, params)));
    }
    sqlifyBinaryOperation(expr, params) {
        return (this.sqlifyExpression(expr.$left, params) +
            ' ' +
            expr.$operator +
            ' ' +
            this.sqlifyExpression(expr.$right, params));
    }
    sqlifyUnaryCompareCondition(expr, params) {
        return this.sqlifyExpression(expr.$expr, params) + ' ' + expr.$operator;
    }
    sqlifyExistsCondition(expr, params) {
        return 'EXISTS(' + this.sqlifySelect(expr.$statement, params) + ')';
    }
    sqlifyUnaryLogicCondition(expr, params) {
        return expr.$operator + ' ' + this.sqlifyCondition(expr.$condition, params);
    }
    sqlifyExpression(expr, params) {
        if (sql_1.Raw.isRaw(expr)) {
            return expr.$sql;
        }
        // 编译标准操作
        if (sql_1.VirtualExpression.isVirtualExpression(expr)) {
            return this.sqlifyExpression(this.translateVirtualSql(expr), params);
        }
        if (sql_1.Literal.isLiteral(expr)) {
            return this.sqlifyLiteral(expr.$value, expr.$dbType);
        }
        if (sql_1.Operation.isOperation(expr)) {
            return this.sqlifyOperation(expr, params);
        }
        if (sql_1.Field.isField(expr)) {
            return this.sqlifyField(expr);
        }
        if (sql_1.GroupExpression.isGroupExpression(expr)) {
            return this.sqlifyGroupExpression(expr, params);
        }
        if (sql_1.ValuedSelect.isValuedSelect(expr)) {
            return this.sqlifyValuedSelect(expr, params);
        }
        if (sql_1.Variant.isVariant(expr)) {
            return this.sqlifyVariantName(expr.$name);
        }
        if (sql_1.Parameter.isParameter(expr)) {
            return this.sqlifyParameter(expr, params);
        }
        if (sql_1.ScalarFuncInvoke.isScalarFuncInvoke(expr)) {
            return this.sqlifyScalarFuncInvoke(expr, params);
        }
        if (sql_1.Case.isCase(expr)) {
            return this.sqlifyCase(expr, params);
        }
        throw this._invalidAST('expression', expr);
    }
    sqlifyInvokeArgs(arg, params) {
        if (sql_1.Star.isStar(arg))
            return this.sqlifyStar(arg);
        if (sql_1.BuiltIn.isBuiltIn(arg))
            return this.sqlifyBuildIn(arg);
        return this.sqlifyExpression(arg, params);
    }
    sqlifyStar(arg) {
        const sql = '';
        if (arg.$table) {
            return `${this.sqlifyRowsetName(arg.$table, true)}.*`;
        }
        return '*';
    }
    /**
     * 函数调用
     * @param {*} invoke
     * @param {*} params
     * @returns
     * @memberof Executor
     */
    sqlifyScalarFuncInvoke(invoke, params) {
        let sql = `${this.sqlifyIdentifier(invoke.$func)}(${(invoke.$args || [])
            .map(v => this.sqlifyInvokeArgs(v, params))
            .join(', ')})`;
        if (invoke.$orders) {
            sql += ' OVER(';
            if (invoke.$partitions) {
                sql +=
                    `PARTITION BY ` +
                        invoke.$partitions
                            .map(expr => this.sqlifyExpression(expr, params))
                            .join(', ');
            }
            sql +=
                'ORDER BY ' +
                    invoke.$orders.map(sort => this.sqlifySort(sort, params)).join(', ');
            sql += ')';
        }
        return sql;
    }
    sqlifyTableFuncInvoke(invoke, params) {
        return `${this.sqlifyIdentifier(invoke.$func)}(${(invoke.$args || [])
            .map(v => this.sqlifyInvokeArgs(v, params))
            .join(', ')})`;
    }
    sqlifyJoin(join, params) {
        if (sql_1.Raw.isRaw(join))
            return join.$sql;
        return ((join.$left ? 'LEFT ' : '') +
            'JOIN ' +
            this.sqlifyFrom(join.$table, params) +
            ' ON ' +
            this.sqlifyCondition(join.$on, params));
    }
    sqlifySort(sort, params) {
        if (sql_1.Raw.isRaw(sort))
            return sort.$sql;
        let sql;
        if (sql_1.SelectColumn.isSelectColumn(sort.$expr)) {
            sql = this.quoted(sort.$expr.$name);
        }
        else {
            sql = this.sqlifyExpression(sort.$expr, params);
        }
        if (sort.$direction)
            sql += ' ' + sort.$direction;
        return sql;
    }
    sqlifySelect(select, params) {
        const { $with, $froms, $top, $joins, $union, $columns, $where, $sorts, $groups, $having, $offset, $limit, $distinct, } = select;
        let sql = '';
        if ($with) {
            sql += this.sqlifyWith($with, params);
        }
        sql += 'SELECT ';
        if ($distinct) {
            sql += 'DISTINCT ';
        }
        if (typeof $top === 'number') {
            sql += `TOP ${$top} `;
        }
        sql += $columns.map(col => this.sqlifyColumn(col, params)).join(', ');
        if ($froms) {
            sql +=
                ' FROM ' +
                    $froms.map(table => this.sqlifyFrom(table, params)).join(', ');
        }
        if ($joins && $joins.length > 0) {
            sql += ' ' + $joins.map(join => this.sqlifyJoin(join, params)).join(' ');
        }
        if ($where) {
            sql += ' WHERE ' + this.sqlifyCondition($where, params);
        }
        if ($groups && $groups.length) {
            sql +=
                ' GROUP BY ' +
                    $groups.map(p => this.sqlifyExpression(p, params)).join(', ');
        }
        if ($having) {
            sql += ' HAVING ' + this.sqlifyCondition($having, params);
        }
        if ($sorts && $sorts.length > 0) {
            sql +=
                ' ORDER BY ' +
                    $sorts.map(sort => this.sqlifySort(sort, params)).join(', ');
        }
        sql += this.sqlifyOffsetLimit(select, params);
        if ($union) {
            sql += ' ' + this.sqlifyUnion($union, params);
        }
        return sql;
    }
    sqlifyOffsetLimit(select, params) {
        let sql = '';
        if (typeof select.$offset === 'number') {
            sql += ` OFFSET ${select.$offset || 0}`;
        }
        if (typeof select.$limit === 'number') {
            sql += ` LIMIT ${select.$limit}`;
        }
        return sql;
    }
    sqlifyFrom(table, params
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ) {
        if (sql_1.Raw.isRaw(table)) {
            return table.$sql;
        }
        if (sql_1.Table.isTable(table) || sql_1.WithSelect.isWithSelect(table)) {
            let sql = '';
            sql += this.sqlifyObjectName(table.$name);
            if (table.$alias)
                sql += ' AS ' + this.quoted(table.$alias);
            return sql;
        }
        if (sql_1.TableVariant.isTableVariant(table)) {
            let sql = '';
            sql += this.sqlifyVariantName(table.$name);
            if (table.$alias)
                sql += ' AS ' + this.quoted(table.$alias);
            return sql;
        }
        // 如果是命名行集
        if (sql_1.NamedSelect.isNamedSelect(table)) {
            return this.sqlifyNamedSelect(table, params);
        }
        if (sql_1.TableFuncInvoke.isTableFuncInvoke(table)) {
            let sql = this.sqlifyTableFuncInvoke(table);
            if (table.$name) {
                sql += ' AS ' + this.quoted(table.$name);
            }
            return sql;
        }
        return this.sqlifyRowsetName(table);
    }
    sqlifyCondition(condition, params
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ) {
        if (sql_1.Raw.isRaw(condition))
            return condition.$sql;
        if (sql_1.ExistsCondition.isExistsCondition(condition)) {
            return this.sqlifyExistsCondition(condition, params);
        }
        if (sql_1.GroupCondition.isGroupCondition(condition)) {
            return this.sqlifyGroupCondition(condition, params);
        }
        if (sql_1.BinaryCompareCondition.isBinaryCompareCondition(condition)) {
            return this.sqlifyBinaryCompareCondition(condition, params);
        }
        if (sql_1.UnaryCompareCondition.isUnaryCompareCondition(condition)) {
            return this.sqlifyUnaryCompareCondition(condition, params);
        }
        if (sql_1.BinaryLogicCondition.isBinaryLogicCondition(condition)) {
            return this.sqlifyBinaryLogicCondition(condition, params);
        }
        if (sql_1.UnaryLogicCondition.isUnaryLogicCondition(condition)) {
            return this.sqlifyUnaryLogicCondition(condition, params);
        }
        if (sql_1.VirtualCondition.isVirtualCondition(condition)) {
            return this.sqlifyCondition(this.translateVirtualSql(condition), params);
        }
        throw this._invalidAST('condition', condition);
    }
    assertAst(value, message = 'AST syntax error.') {
        if (!value) {
            throw new Error('AST syntax error:' + message);
        }
    }
    sqlifyInsert(insert, params) {
        const { $table, $values, $fields, $with } = insert;
        let sql = '';
        if ($with) {
            sql += this.sqlifyWith($with, params);
        }
        sql += 'INSERT INTO ';
        if ($table.$alias) {
            throw new Error('Insert statements do not allow aliases on table.');
        }
        sql += this.sqlifyRowsetName($table);
        if ($fields) {
            sql +=
                '(' +
                    $fields.map(field => this.sqlifyInsertField(field)).join(', ') +
                    ')';
        }
        this.assertAst($values);
        if (Array.isArray($values)) {
            sql += ' VALUES';
            sql += $values
                .map(row => '(' +
                row.map(expr => this.sqlifyExpression(expr, params)).join(', ') +
                ')')
                .join(', ');
        }
        else {
            sql += ' ' + this.sqlifySelect($values, params);
        }
        return sql;
    }
    sqlifyAssignment(assign, params) {
        const { left, right } = assign;
        return (this.sqlifyExpression(left, params) +
            ' = ' +
            this.sqlifyExpression(right, params));
    }
    sqlifyVariantDeclare(varDec) {
        return (this.sqlifyVariantName(varDec.$name) +
            ' ' +
            this.sqlifyType(varDec.$dbType));
    }
    sqlifyUpdate(update, params
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ) {
        const { $table, $sets, $with, $where, $froms, $joins } = update;
        (0, assert_1.default)($table, 'table is required by update statement');
        (0, assert_1.default)($sets, 'set statement un declared');
        let sql = '';
        if ($with) {
            sql += this.sqlifyWith($with, params);
        }
        sql += 'UPDATE ';
        sql += this.sqlifyRowsetName($table);
        sql +=
            ' SET ' +
                $sets
                    .map(assign => this.sqlifyExpression(assign.left, params) +
                    ' = ' +
                    this.sqlifyExpression(assign.right, params))
                    .join(', ');
        if ($froms && $froms.length > 0) {
            sql +=
                ' FROM ' +
                    $froms.map(table => this.sqlifyFrom(table, params)).join(', ');
        }
        if ($joins && $joins.length > 0) {
            sql += ' ' + $joins.map(join => this.sqlifyJoin(join, params)).join(' ');
        }
        if ($where) {
            sql += ' WHERE ' + this.sqlifyCondition($where, params);
        }
        return sql;
    }
    sqlifyDelete(del, params) {
        const { $table, $froms, $joins, $where, $with } = del;
        let sql = '';
        if ($with) {
            sql += this.sqlifyWith($with, params);
        }
        sql += 'DELETE ';
        if ($table)
            sql += this.sqlifyRowsetName($table);
        if ($froms && $froms.length > 0) {
            sql +=
                ' FROM ' +
                    $froms.map(table => this.sqlifyFrom(table, params)).join(', ');
        }
        if ($joins) {
            sql += $joins.map(join => this.sqlifyJoin(join, params)).join(' ');
        }
        if ($where) {
            sql += ' WHERE ' + this.sqlifyCondition($where, params);
        }
        return sql;
    }
}
exports.Sqlifier = Sqlifier;
