import { DbType, Condition, If, While, Break, Continue, XExpression, Return, Statement, Document, Scalar, ExpandScalar, Integer, DbTypeFromScalar, Literal, Variant, XRowsets, Field, Select, RowObject, TableVariantBuilder, TableVariantMember, XTableVariant, XObjectName, XTable, Func, Procedure, BuiltIn, Star, XRowset, CompatiblifyTuple, Execute, Assignable, Assignment, TableVariant, Declare, When, Case, WithSelect, XWithSelect, SelectAliasObject, With, CreateTableBuilder, AlterTableBuilder, AlterView, CreateIndex, CreateProcedureBuilder, AlterProcedureBuilder, CreateFunctionBuilder, AlterFunctionBuilder, DropIndex, Annotation, Insert, Update, Delete, Block, AlterDatabase, Parameter, ScalarFromDbType, Sequence, Expression, XTables, ColumnsOf, Numeric, SelectAction, CreateDatabaseBuilder, CreateSequenceBuilder, CreateViewBuilder, ScalarFuncInvoke, DropTableBuilder } from './sql';
import { CreateSchemaBuilder, DropDatabaseBuilder, DropFunctionBuilder, DropProcedureBuilder, DropSchemaBuilder, DropSequenceBuilder, DropViewBuilder } from './sql/statement';
import { VirtualOperation } from './virtual-operation';
export interface SqlBuilder extends VirtualOperation {
    readonly type: typeof DbType;
    if(condition: Condition): If;
    while(condition: Condition): While;
    break(): Break;
    continue(): Continue;
    return(value?: XExpression): Return;
    /**
     * 创建一个SQL文档
     * @param statements 文档代码
     */
    doc(statements: Statement[]): Document;
    doc(...statements: Statement[]): Document;
    group(condition: Condition): Condition;
    group<T extends Scalar>(expr: XExpression<T>): Expression<T>;
    /**
     * 负号运算符 -
     */
    neg(expr: XExpression<number>): Expression<ExpandScalar<number>>;
    /**
     * 字符串连接运算
     */
    concat(...strs: [
        XExpression<string>,
        XExpression<string>,
        ...XExpression<string>[]
    ]): Expression<string>;
    /**
     * 算术运算 +
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    add<T extends Numeric>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 算术运算 -
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    sub<T extends Numeric>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 算术运算 *
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    mul<T extends Numeric>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 算术运算 /
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    div<T extends Numeric>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 算术运算 %
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    mod<T extends Numeric>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    xor<T extends Integer>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 位算术运算 << 仅支持整型
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    shl<T extends Integer>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 位算术运算 >>
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    shr<T extends Integer>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    literal<T extends Scalar>(value: T, dbType?: DbTypeFromScalar<T>): Literal<ExpandScalar<T>>;
    var<T extends Scalar, N extends string = string>(name: N, type: DbTypeFromScalar<T>): Variant<ExpandScalar<T>, N>;
    /**
     * 创建一个字段
     */
    field<T extends Scalar, N extends string>(name: N, rowset?: XRowsets): Field<T, N>;
    /**
     * 将多个查询条件通过 AND 合并成一个大查询条件
     * @param conditions 查询条件列表
     * @returns 返回逻辑表达式
     */
    and(conditions: Condition[]): Condition;
    and(...conditions: [Condition, Condition, ...Condition[]]): Condition;
    /**
     * 位算术运算 &
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    and<T extends Integer>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * 将多个查询条件通过 OR 合并成一个
     * @param conditions 查询条件列表
     * @returns 返回逻辑表达式
     */
    or(conditions: Condition[]): Condition;
    or(...conditions: [Condition, Condition, ...Condition[]]): Condition;
    /**
     * 位算术运算 |
     * @param left 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    or<T extends Integer>(left: XExpression<T>, right: XExpression<T>): Expression<T>;
    /**
     * Not 逻辑运算
     * @param condition
     */
    not(condition: Condition): Condition;
    /**
     * 位算术运算 ~
     * @param value 左值
     * @param right 右值
     * @returns 返回算术运算表达式
     */
    not<T extends Integer>(value: XExpression<T>): Expression<T>;
    /**
     * 比较运算 IS NULL
     * @returns 返回比较运算符
     * @param expr 表达式
     */
    isNull(expr: XExpression<Scalar>): Condition;
    /**
     * 比较运算 IS NOT NULL
     * @param expr 表达式
     * @returns 返回比较运算符
     */
    isNotNull(expr: XExpression<Scalar>): Condition;
    eq<T extends Scalar>(left: XExpression<T>, right: XExpression<T>): Condition;
    /**
     * 比较运算 <>
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    neq<T extends Scalar>(left: XExpression<T>, right: XExpression<T>): Condition;
    lt<T extends Scalar>(left: XExpression<T>, right: XExpression<T>): Condition;
    /**
     * 比较运算 <=
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    lte<T extends Scalar>(left: XExpression<T>, right: XExpression<T>): Condition;
    /**
     * 比较运算 >
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    gt<T extends Scalar>(left: XExpression<T>, right: XExpression<T>): Condition;
    /**
     * 比较运算 >=
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    gte<T extends Scalar>(left: XExpression<T>, right: XExpression<T>): Condition;
    /**
     * 比较运算 LIKE
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    like(left: XExpression<string>, right: XExpression<string>): Condition;
    /**
     * 比较运算 NOT LIKE
     * @param left 左值
     * @param right 右值
     * @returns 返回比较运算对比条件
     */
    notLike(left: XExpression<string>, right: XExpression<string>): Condition;
    /**
     * 比较运算 IN
     * @param left 左值
     * @param values 要比较的值列表
     * @returns 返回比较运算对比条件
     */
    in<T extends Scalar>(left: XExpression<T>, select: Select<any>): Condition;
    in<T extends Scalar>(left: XExpression<T>, values: XExpression<T>[]): Condition;
    in<T extends Scalar>(left: XExpression<T>, values: XExpression<T>[] | Select<any>): Condition;
    /**
     * 比较运算 NOT IN
     * @param left 左值
     * @param values 要比较的值列表
     * @returns 返回比较运算对比条件
     */
    notIn<T extends Scalar>(left: XExpression<T>, values: XExpression<T>[]): Condition;
    /**
     * 判断是否存在
     * @param select 查询语句
     */
    exists(select: Select<any>): Condition;
    table<T extends RowObject = any>(name: string, members: ((builder: TableVariantBuilder) => TableVariantMember[]) | TableVariantMember[]): XTableVariant<T>;
    table<T extends RowObject = any>(name: XObjectName, builtIn?: boolean): XTable<T>;
    /**
     * 声明一个函数
     */
    func(name: XObjectName, builtIn?: boolean): Func;
    /**
     * 创建一个可供调用的存储过程函数
     */
    proc<R extends Scalar = number, O extends RowObject[] = never, N extends string = string>(name: XObjectName<N>, builtIn?: boolean): Procedure<R, O, N>;
    builtIn<N extends string>(name: N): BuiltIn<N>;
    star(): Star<any>;
    invokeAsTable<T extends RowObject = any>(func: XObjectName | Func<string>, args: (Star | BuiltIn | XExpression<Scalar>)[]): XRowset<T>;
    invokeAsScalar<T extends Scalar = any>(func: XObjectName | Func<string>, args: (Star | BuiltIn | XExpression<Scalar>)[]): ScalarFuncInvoke<T>;
    makeInvoke(type: 'table', name: XObjectName, builtIn?: boolean): (...args: (Star | BuiltIn | XExpression | undefined)[]) => XRowset<any>;
    makeInvoke<R extends RowObject, A extends (Scalar | Star | BuiltIn | undefined)[] = []>(type: 'table', name: XObjectName, builtIn?: boolean): (...args: CompatiblifyTuple<A>) => XRowset<R>;
    makeInvoke(type: 'scalar', name: XObjectName, builtIn?: boolean): (...args: (Star | BuiltIn | XExpression | undefined)[]) => Expression<any>;
    makeInvoke<R extends Scalar, A extends (Scalar | Star | BuiltIn | undefined)[] = []>(type: 'scalar', name: XObjectName, builtIn?: boolean): (...args: CompatiblifyTuple<A>) => Expression<R>;
    /**
     * 创建一个可供JS调用的存储过程
     */
    makeExec(name: XObjectName, builtIn?: boolean): (...args: (BuiltIn | Scalar | XExpression | undefined)[]) => Execute<any, any>;
    makeExec<R extends Scalar = number, A extends (BuiltIn | Scalar | XExpression | undefined)[] = [], O extends RowObject[] = []>(name: XObjectName, builtIn?: boolean): (...args: CompatiblifyTuple<A>) => Execute<R, O>;
    /**
     * 赋值语句
     * @param left 左值
     * @param right 右值
     */
    set<T extends Scalar = any>(left: Assignable<T>, right: XExpression<T>): Assignment<T>;
    /**
     * 变量声明
     * @param declares 变量列表
     */
    declare(...members: (Variant<any> | TableVariant<any>)[] | [(Variant<any> | TableVariant<any>)[]]): Declare;
    /**
     * WHEN 语句块
     * @param expr
     * @param value
     */
    when<T extends Scalar>(expr: XExpression<Scalar>, value: XExpression<T>): When<T>;
    case<E extends Scalar>(expr?: XExpression<E>): Case<any, E>;
    /**
     * With语句
     */
    with(...rowsets: WithSelect[] | [WithSelect[]] | XWithSelect[] | [XWithSelect[]] | [SelectAliasObject]): With;
    union<T extends RowObject = any>(...selects: Select<T>[]): Select<T>;
    unionAll<T extends RowObject = any>(...selects: Select<T>[]): Select<T>;
    readonly createTable: CreateTableBuilder;
    readonly alterTable: AlterTableBuilder;
    createView: CreateViewBuilder;
    alterView<T extends RowObject = any, N extends string = string>(name: XObjectName<N>): AlterView<T, N>;
    createIndex(name: string): CreateIndex;
    readonly createProcedure: CreateProcedureBuilder;
    readonly alterProcedure: AlterProcedureBuilder;
    readonly createFunction: CreateFunctionBuilder;
    readonly alterFunction: AlterFunctionBuilder;
    readonly dropProcedure: DropProcedureBuilder;
    readonly dropTable: DropTableBuilder;
    readonly dropView: DropViewBuilder;
    readonly dropFunction: DropFunctionBuilder;
    dropIndex<N extends string>(table: XObjectName, name: N): DropIndex<N>;
    annotation(...text: string[]): Annotation;
    note(text: string): Annotation;
    /**
     * 插入至表,into的别名
     * @param table
     * @param fields
     */
    insert<T extends RowObject = any>(table: XTables<T>, fields?: ColumnsOf<T>[] | Field<Scalar, ColumnsOf<T>>[]): Insert<T>;
    /**
     * 更新一个表格
     * @param table
     */
    update<T extends RowObject = any>(table: XTables<T>): Update<T>;
    /**
     * 删除一个表格
     * @param table 表格
     */
    delete<T extends RowObject = any>(table: XTables<T>): Delete<T>;
    readonly select: SelectAction;
    raw(sql: string): any;
    block(...args: Statement[] | [Statement[]]): Block;
    execute<R extends Scalar = any, O extends RowObject[] = []>(proc: XObjectName | Procedure<R, O, string>, params?: (BuiltIn | XExpression)[]): Execute<R, O>;
    readonly createDatabase: CreateDatabaseBuilder;
    readonly createSchema: CreateSchemaBuilder;
    readonly dropSchema: DropSchemaBuilder;
    alterDatabase(name: string): AlterDatabase;
    readonly dropDatabase: DropDatabaseBuilder;
    readonly createSequence: CreateSequenceBuilder;
    readonly dropSequence: DropSequenceBuilder;
    /**
     * input 参数
     */
    input<T extends Scalar, N extends string>(name: N, value: T): Parameter<ExpandScalar<T>, N>;
    input<T extends DbType, N extends string>(name: N, type: T, value?: ScalarFromDbType<T>): Parameter<ExpandScalar<ScalarFromDbType<T>>, N>;
    /**
     * output参数
     */
    output<T extends DbType, N extends string>(name: N, type: T, value?: ScalarFromDbType<T>): Parameter<ExpandScalar<ScalarFromDbType<T>>, N>;
    /**
     * 创建一个输入输出参数
     */
    inoutput<T extends DbType, N extends string>(name: N, type: T, value?: ScalarFromDbType<T>): Parameter<ExpandScalar<ScalarFromDbType<T>>, N>;
    sequence<T extends Numeric>(name: XObjectName): Sequence<T>;
}
export declare const SQL: SqlBuilder;
