import { Sql, SQL_SYMBOLE } from '../sql';
import type { Integer, Scalar } from '../scalar';
import type { DbType, ScalarFromDbType } from '../db-type';
/**
 * 可兼容JS常量的表达式
 */
export declare type XExpression<T extends Scalar = Scalar> = Expression<T> | T;
/**
 * 表达式基类，抽象类，
 * 所有表达式类均从该类型继承，
 * 可以直接使用 instanceof 来判断是否为expression
 */
export declare abstract class Expression<T extends Scalar = Scalar> extends Sql implements Expression {
    $tag: SQL_SYMBOLE.EXPRESSION;
    /**
     * 字符串连接运算
     */
    concat(expr: XExpression<string>): Expression<string>;
    /**
     * 加法运算，返回数值，如果是字符串相连接，请使用join函数
     */
    add(expr: XExpression<T>): Expression<T>;
    /**
     * 减法运算
     */
    sub(expr: XExpression<T>): Expression<T>;
    /**
     * 乘法运算
     * @param expr 要与当前表达式相乘的表达式
     */
    mul(expr: XExpression<T>): Expression<T>;
    /**
     * 除法运算
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    div(expr: XExpression<T>): Expression<T>;
    /**
     * 取模运算 %
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    mod(expr: XExpression<T>): Expression<T>;
    /**
     * 位运算 &
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    and(expr: XExpression<Integer>): Expression<Integer>;
    /**
     * 位运算 |
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    or(expr: XExpression<Integer>): Expression<Integer>;
    /**
     * 位运算 ^
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    xor(expr: XExpression<Integer>): Expression<Integer>;
    /**
     * 位运算 <<
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    shl(expr: XExpression<Integer>): Expression<T>;
    /**
     * 位运算 >>
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    shr(bits: XExpression<Integer>): Expression<T>;
    /**
     * 比较是否相等 =
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    eq(expr: XExpression<T>): Condition;
    /**
     * 比较是否不等于 <>
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    neq(expr: XExpression<T>): Condition;
    /**
     * 比较是否小于 <
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    lt(expr: XExpression<T>): Condition;
    /**
     * 比较是否小于等于 <=
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    lte(expr: XExpression<T>): Condition;
    /**
     * 比较是否大于 >
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    gt(expr: XExpression<T>): Condition;
    /**
     * 比较是否小于等于 >=
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    gte(expr: XExpression<T>): Condition;
    /**
     * 比较是相像 LIKE
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    like(expr: XExpression<string>): Condition;
    /**
     * 比较是否不想像 NOT LIKE
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    notLike(expr: XExpression<string>): Condition;
    /**
     * 比较是否不包含于 IN
     * @param values 要与当前表达式相比较的表达式数组
     * @returns 返回对比条件表达式
     */
    in(select: Select<any>): Condition;
    in(values: XExpression<T>[]): Condition;
    in(...values: XExpression<T>[]): Condition;
    /**
     * 比较是否不包含于 NOT IN
     * @param values 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    notIn(select: Select<any>): Condition;
    notIn(values: XExpression<T>[]): Condition;
    notIn(...values: XExpression<T>[]): Condition;
    /**
     * 比较是否为空 IS NULL
     * @returns 返回对比条件表达式
     */
    isNull(): Condition;
    /**
     * 比较是否为空 IS NOT NULL
     * @returns 返回对比条件表达式
     */
    isNotNull(): Condition;
    /**
     * 正序
     * @returns 返回对比条件表达式
     */
    asc(): Sort;
    /**
     * 倒序
     * @returns 返回对比条件表达式
     */
    desc(): Sort;
    /**
     * 将表达式转换为列，并指定列名
     */
    as<N extends string>(name: N): SelectColumn<T, N>;
    /**
     * 将当前表达式转换为指定的类型
     */
    to<T extends DbType>(type: T): Expression<ScalarFromDbType<T>>;
    static isExpression(object: any): object is Expression;
    static ensureExpression<T extends Scalar>(value: XExpression<T>): Expression<T>;
}
import { Condition } from '../condition';
import { Select, SelectColumn, Sort } from '../statement';
