"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Expression = void 0;
const sql_1 = require("../sql");
/**
 * 表达式基类，抽象类，
 * 所有表达式类均从该类型继承，
 * 可以直接使用 instanceof 来判断是否为expression
 */
class Expression extends sql_1.Sql {
    constructor() {
        super(...arguments);
        this.$tag = sql_1.SQL_SYMBOLE.EXPRESSION;
    }
    /**
     * 字符串连接运算
     */
    concat(expr) {
        return sql_builder_1.SQL.concat(this, expr);
    }
    /**
     * 加法运算，返回数值，如果是字符串相连接，请使用join函数
     */
    add(expr) {
        return sql_builder_1.SQL.add(this, expr);
    }
    /**
     * 减法运算
     */
    sub(expr) {
        return sql_builder_1.SQL.sub(this, expr);
    }
    /**
     * 乘法运算
     * @param expr 要与当前表达式相乘的表达式
     */
    mul(expr) {
        return sql_builder_1.SQL.mul(this, expr);
    }
    /**
     * 除法运算
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    div(expr) {
        return sql_builder_1.SQL.div(this, expr);
    }
    /**
     * 取模运算 %
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    mod(expr) {
        return sql_builder_1.SQL.mod(this, expr);
    }
    /**
     * 位运算 &
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    and(expr) {
        return sql_builder_1.SQL.and(this, expr);
    }
    /**
     * 位运算 |
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    or(expr) {
        return sql_builder_1.SQL.or(this, expr);
    }
    // /**
    //  * 位运算 ~
    //  * @param expr 要与当前表达式相除的表达式
    //  * @returns 返回运算后的表达式
    //  */
    // not(): Expression<T> {
    //   return UnaryOperation.not(this as any) as any
    // }
    /**
     * 位运算 ^
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    xor(expr) {
        return sql_builder_1.SQL.xor(this, expr);
    }
    /**
     * 位运算 <<
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    shl(expr) {
        return sql_builder_1.SQL.shl(this, expr);
    }
    /**
     * 位运算 >>
     * @param expr 要与当前表达式相除的表达式
     * @returns 返回运算后的表达式
     */
    shr(bits) {
        return sql_builder_1.SQL.shr(this, bits);
    }
    /**
     * 比较是否相等 =
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    eq(expr) {
        return sql_builder_1.SQL.eq(this, expr);
    }
    /**
     * 比较是否不等于 <>
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    neq(expr) {
        return sql_builder_1.SQL.neq(this, expr);
    }
    /**
     * 比较是否小于 <
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    lt(expr) {
        return sql_builder_1.SQL.lt(this, expr);
    }
    /**
     * 比较是否小于等于 <=
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    lte(expr) {
        return sql_builder_1.SQL.lte(this, expr);
    }
    /**
     * 比较是否大于 >
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    gt(expr) {
        return sql_builder_1.SQL.gt(this, expr);
    }
    /**
     * 比较是否小于等于 >=
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    gte(expr) {
        return sql_builder_1.SQL.gte(this, expr);
    }
    /**
     * 比较是相像 LIKE
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    like(expr) {
        return sql_builder_1.SQL.like(this, expr);
    }
    /**
     * 比较是否不想像 NOT LIKE
     * @param expr 要与当前表达式相比较的表达式
     * @returns 返回对比条件表达式
     */
    notLike(expr) {
        return sql_builder_1.SQL.notLike(this, expr);
    }
    in(...values) {
        if (values.length === 1 &&
            (statement_1.Select.isSelect(values[0]) || Array.isArray(values[0]))) {
            return sql_builder_1.SQL.in(this, values[0]);
        }
        return sql_builder_1.SQL.in(this, values);
    }
    notIn(...values) {
        if (values.length === 1 &&
            (statement_1.Select.isSelect(values[0]) || Array.isArray(values[0]))) {
            return sql_builder_1.SQL.notIn(this, values[0]);
        }
        return sql_builder_1.SQL.notIn(this, values);
    }
    /**
     * 比较是否为空 IS NULL
     * @returns 返回对比条件表达式
     */
    isNull() {
        return sql_builder_1.SQL.isNull(this);
    }
    /**
     * 比较是否为空 IS NOT NULL
     * @returns 返回对比条件表达式
     */
    isNotNull() {
        return sql_builder_1.SQL.isNotNull(this);
    }
    /**
     * 正序
     * @returns 返回对比条件表达式
     */
    asc() {
        return new statement_1.Sort(this, 'ASC');
    }
    /**
     * 倒序
     * @returns 返回对比条件表达式
     */
    desc() {
        return new statement_1.Sort(this, 'DESC');
    }
    /**
     * 将表达式转换为列，并指定列名
     */
    as(name) {
        return new statement_1.SelectColumn(name, this);
    }
    // /**
    //  * 将本表达式括起来
    //  */
    // group(): Expression<T> {
    //   return new GroupExpression(this);
    // }
    /**
     * 将当前表达式转换为指定的类型
     */
    to(type) {
        return sql_builder_1.SQL.convert(this, type);
    }
    static isExpression(object) {
        return object?.$tag === sql_1.SQL_SYMBOLE.EXPRESSION;
    }
    static ensureExpression(value) {
        return Expression.isExpression(value)
            ? value
            : literal_1.Literal.ensureLiteral(value);
    }
}
exports.Expression = Expression;
const statement_1 = require("../statement");
const literal_1 = require("./literal");
const sql_builder_1 = require("../../sql-builder");
