"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Insert = void 0;
const assert_1 = __importDefault(require("assert"));
const rowset_1 = require("../../rowset");
const expression_1 = require("../../expression");
const scalar_1 = require("../../scalar");
const statement_1 = require("../statement");
const select_1 = require("./select");
/**
 * Insert 语句
 */
class Insert extends statement_1.Statement {
    /**
     * 构造函数
     */
    constructor(table, fields) {
        super();
        this.$identityInsert = false;
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$kind = statement_1.STATEMENT_KIND.INSERT;
        this.$identityInsert = false;
        this.$table =
            rowset_1.Table.isTable(table) || rowset_1.TableVariant.isTableVariant(table)
                ? table
                : rowset_1.Table.create(table);
        if (this.$table.$alias) {
            throw new Error('Insert statements do not allow aliases on table.');
        }
        if (fields) {
            this.fields(fields);
        }
    }
    /**
     * 在插入数据时开启标识列插入，即IdentityInsert On
     * @returns
     */
    withIdentity() {
        this.$identityInsert = true;
        return this;
    }
    /**
     * 指定插入字段，当使用数组值列表时，必须指定字段名
     */
    fields(...fields) {
        if (Array.isArray(fields[0])) {
            fields = fields[0];
        }
        if (typeof fields[0] === 'string') {
            this.$fields = fields.map(field => this.$table.$field(field));
        }
        else {
            this.$fields = fields;
        }
    }
    // 将值推入内部，最大1000条。
    _values(items, FIELDS_SEARCH_MAX_SIZE = 1000) {
        if (!this.$fields) {
            const existsFields = {};
            for (let i = 0; i < Math.min(FIELDS_SEARCH_MAX_SIZE, items.length); i++) {
                const item = items[i];
                Object.keys(item).forEach(field => {
                    if (!existsFields[field])
                        existsFields[field] = true;
                });
            }
            this.$fields = Object.keys(existsFields).map(fieldName => {
                return this.$table.$field(fieldName);
            });
        }
        const fields = this.$fields.map(field => field.$name);
        return items.map((item) => {
            return fields.map(fieldName => expression_1.Expression.ensureExpression(item[fieldName] ?? null));
        });
    }
    values(...args) {
        (0, assert_1.default)(!this.$values, 'Values is exists.');
        (0, assert_1.default)(args.length > 0 && !(args.length === 1 && !args[0]), 'rows must more than one elements.');
        // args: [Select<T>]
        if (args.length === 1 && select_1.Select.isSelect(args[0])) {
            this.$values = args[0];
            return this;
        }
        // args: [InputObject<T>]
        if (!Array.isArray(args[0]) &&
            !this.$fields &&
            args.length === 1 &&
            isInputObject(args[0])) {
            this.$values = this._values(args);
            return this;
        }
        // args: InputObject<T>[]
        if (!this.$fields && isInputObjectArray(args)) {
            this.$values = this._values(args);
            return this;
        }
        // args: [InputObject<T>[]]
        if (Array.isArray(args[0]) &&
            !this.$fields &&
            isInputObjectArray(args[0])) {
            this.$values = this._values(args[0]);
            return this;
        }
        (0, assert_1.default)(this.$fields, `Must call .fields() first, when use array values.`);
        // args: [Scalar[][]]
        if (Array.isArray(args[0]) &&
            args.length === 1 &&
            isRowDataArray(args[0], this.$fields.length)) {
            this.$values = args[0].map(row => {
                (0, assert_1.default)(row.length === this.$fields?.length, `Values count must same with fields.`);
                return row.map(exp => expression_1.Expression.ensureExpression(exp));
            });
            return this;
        }
        // args: Scalar[][] | [Scalar[]]
        // if (isRowDataArray(args, this.$fields.length)) {
        this.$values = args.map(row => {
            (0, assert_1.default)(row.length === this.$fields?.length, `Values count must same with fields.`);
            return row.map(exp => expression_1.Expression.ensureExpression(exp));
        });
        return this;
        // }
    }
    static isInsert(object) {
        return (statement_1.Statement.isStatement(object) && object.$kind === statement_1.STATEMENT_KIND.INSERT);
    }
}
exports.Insert = Insert;
Insert.isRowDataArray = isRowDataArray;
function isInputObject(object) {
    return (typeof object === 'object' &&
        !Array.isArray(object) &&
        !(0, scalar_1.isBaseScalar)(object));
}
/**
 * 判断是否为InputObject数组
 */
function isInputObjectArray(datas, MAX_SEARCH_SIZE = 100) {
    for (let i = 0; i < Math.min(MAX_SEARCH_SIZE, datas.length); i++) {
        if (!isInputObject(datas[i])) {
            return false;
        }
    }
    return true;
}
/**
 * 判断一个数组是否是一个值列表数组，即Scalar[][];
 */
function isRowDataArray(items, 
/**
 * 值数量
 */
valueLength, MAX_SEARCH_SIZE = 10) {
    if (!Array.isArray(items))
        return false;
    if (items.length === 0)
        return false;
    // 每一行的值数量与字段数量匹配，则表示是值列表数组
    for (let i = 0; i < Math.min(MAX_SEARCH_SIZE, items.length); i++) {
        if (!(Array.isArray(items[i]) && items[i].length === valueLength))
            return false;
    }
    return true;
}
