"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Select = void 0;
const assert_1 = __importDefault(require("assert"));
const scalar_1 = require("../../scalar");
const star_1 = require("../../object/star");
const expression_1 = require("../../expression");
const rowset_1 = require("../../rowset");
const statement_1 = require("../statement");
const select_action_1 = require("./common/select-action");
const fromable_1 = require("./common/fromable");
const select_column_1 = require("./select-column");
const sort_1 = require("./sort");
const union_1 = require("./union");
/**
 * SELECT查询，传入的表达式会被自动命名为 '#column_' + i;
 */
class Select extends fromable_1.Fromable {
    constructor(...columns) {
        super();
        this.$tag = statement_1.STATEMENT_TAG.EXECUTABLE;
        this.$kind = statement_1.STATEMENT_KIND.SELECT;
        (0, assert_1.default)(columns.length > 0, 'Must select one or more columns by Select statement.');
        if (columns.length === 1 &&
            !star_1.Star.isStar(columns[0]) &&
            !select_column_1.SelectColumn.isSelectColumn(columns[0]) &&
            !expression_1.Expression.isExpression(columns[0]) &&
            !(0, scalar_1.isBaseScalar)(columns[0])) {
            const results = columns[0];
            this.$columns = Object.entries(results).map(([name, expr]) => {
                if (expression_1.Literal.isLiteral(expr) && !expr.$dbType) {
                    expr.fitType();
                }
                return new select_column_1.SelectColumn(name, 
                // 选择项需要转换成目标类型，避免返回类型不对的问题
                expression_1.Expression.ensureExpression(expr));
            });
            return;
        }
        let i = 0;
        // 实例化
        this.$columns = columns.map(item => {
            if (expression_1.Field.isField(item)) {
                return item.as(item.$name);
            }
            if (star_1.Star.isStar(item) || select_column_1.SelectColumn.isSelectColumn(item)) {
                return item;
            }
            return new select_column_1.SelectColumn(select_action_1.COLUMN_NAME_PREEFIX + (++i).toString(), expression_1.Expression.ensureExpression(item));
        });
    }
    /**
     * 去除重复的
     */
    distinct() {
        this.$distinct = true;
        return this;
    }
    /**
     * TOP
     * @param rows 行数
     */
    top(rows) {
        // assert(typeof this.$top === 'undefined', 'top is declared')
        this.$top = rows;
        return this;
    }
    orderBy(...args) {
        // assert(!this.$orders, 'order by clause is declared')
        (0, assert_1.default)(args.length > 0, 'must have one or more order basis');
        let ds;
        // 如果传入的是对象类型
        if (Array.isArray(args[0])) {
            ds = args[0];
        }
        else {
            ds = args;
        }
        if (ds.length === 1 && !sort_1.Sort.isSort(ds[0])) {
            const obj = ds[0];
            this.$sorts = Object.entries(obj).map(([expr, direction]) => new sort_1.Sort(expr, direction));
            return this;
        }
        this.$sorts = ds.map(item => {
            if (sort_1.Sort.isSort(item))
                return item;
            if ((0, scalar_1.isBaseScalar)(item) || expression_1.Expression.isExpression(item)) {
                return new sort_1.Sort(item);
            }
            throw new Error(`Not sortable object: ${item}.`);
        });
        return this;
    }
    /**
     * 分组查询
     * @param groups
     */
    groupBy(...groups) {
        this.$groups = groups.map(expr => expression_1.Expression.ensureExpression(expr));
        return this;
    }
    /**
     * Having 子句
     * @param condition
     */
    having(condition) {
        (0, assert_1.default)(!this.$having, 'having is declared');
        (0, assert_1.default)(this.$groups, 'Syntax error, group by is not declared.');
        this.$having = condition;
        return this;
    }
    /**
     * 偏移数
     * @param rows
     */
    offset(rows) {
        this.$offset = rows;
        return this;
    }
    /**
     * 限定数
     * @param rows
     */
    limit(rows) {
        // assert(typeof rows === 'number', 'The argument rows must type of Number')
        this.$limit = rows;
        return this;
    }
    /**
     * 合并查询
     */
    union(select, all = false) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        let sel = this;
        // 查找最末端的select，将union关联到最末端select语句中
        while (sel.$union) {
            sel = sel.$union.$select;
        }
        sel.$union = new union_1.Union(select, all);
        return this;
    }
    unionAll(select) {
        return this.union(select, true);
    }
    /**
     * 将本次查询，转换为Table行集
     * @param alias
     */
    as(alias) {
        return rowset_1.NamedSelect.create(this, alias);
    }
    asWith(name) {
        return rowset_1.WithSelect.create(name, this);
    }
    /**
     * 将本次查询结果转换为值
     */
    asValue() {
        return new expression_1.ValuedSelect(this);
    }
    asColumn(name) {
        return this.asValue().as(name);
    }
    static isSelect(object) {
        return (statement_1.Statement.isStatement(object) && object.$kind === statement_1.STATEMENT_KIND.SELECT);
    }
}
exports.Select = Select;
