/// <reference types="node" />
import 'colors';
import { DbContext, DatabaseSchema } from '../orm';
import { Command } from '../core';
import { MigrateBuilder } from './migrate-builder';
import './types';
import { Writable } from 'stream';
import { Connection } from '../core/base/connection';
export interface Migrate {
    up(builder: MigrateBuilder, dialect: string | symbol): Promise<void> | void;
    down(builder: MigrateBuilder, dialect: string | symbol): Promise<void> | void;
}
declare module '../core/config' {
    interface LubeConfig {
        migrateDir?: string;
    }
}
interface MigrateInfo {
    id: string;
    name: string;
    timestamp: string;
    path: string;
    kind: 'typescript' | 'javascript';
    hash?: string;
    snapshotPath: string;
    index: number;
}
export declare const LUBE_MIGRATE_TABLE_NAME = "__LubeMigrate";
export declare function outputCommand(cmd: Command, writer: Writable): void;
/**
 * 迁移命令行
 */
declare class MigrateCliClass {
    readonly migrateDir: string;
    private _targetSchemaName;
    constructor(context?: string | DbContext, migrateDir?: string);
    connection: Connection;
    dbContext: DbContext;
    private provider;
    private migrateBuilder;
    private schemaLoader;
    private init;
    private runMigrate;
    private ensureSchema;
    /**
     * 数据库是否存在
     * @returns
     */
    private existsTargetDatabase;
    private _script;
    private _migrateList?;
    dispose(): Promise<void>;
    /**
     * 获取数据库当前迁移版本
     * @returns
     */
    getDbMigrate(): Promise<MigrateInfo | undefined>;
    private getLastMigrate;
    /**
     * 创建数据库
     */
    ensureTargetDatabase(): Promise<void>;
    /**
     * 如果数据库已经存在，删除目标数据库
     */
    dropTargetDatabase(): Promise<void>;
    private getTimestamp;
    /**
     * 将数据库架构与当前架构进行对比，并生成新的迁移文件
     * @param name
     */
    add(name?: string, notResolverType?: boolean): Promise<MigrateInfo>;
    findMigrate(name: string): Promise<MigrateInfo | undefined>;
    getMigrate(nameOrId: string): Promise<MigrateInfo>;
    /**
     * 获取数据库架构
     * @returns
     */
    getDbSchema(): Promise<DatabaseSchema | undefined>;
    /**
     * 获取实体的架构
     * @returns
     */
    getEntitySchema(): DatabaseSchema;
    /**
     * 目标数据库名称，连接字符串中指定的数据库（优先） 或者 metadata中指定的数据库
     */
    targetDatabase: string;
    get targetSchemaName(): string;
    /**
     * 更新到指定迁移
     * 无论是否比指定迁移更新
     * @param name
     */
    update(name?: string, output?: Writable): Promise<void>;
    script(options?: {
        start?: string;
        end?: string;
        outputPath?: string;
    }): Promise<string>;
    /**
     * 列出当前所有迁移
     */
    private _list;
    /**
     * 列出迁移文件
     */
    list(): Promise<void>;
    /**
     * 生成迁移文件代码
     * @param name
     * @param source
     * @param target
     * @param metadata
     * @returns
     */
    private generateMigrate;
    private generateMigrateClass;
    /**
     * 同步架构
     */
    sync(output?: Writable): Promise<void>;
    /**
     * 生成快照
     */
    snapshotAll(): Promise<void>;
}
export declare type MigrateCli = MigrateCliClass;
export declare type MigrateCliConstructor = {
    new (context?: string | DbContext, migrateDir?: string): Promise<MigrateCli>;
};
export declare const MigrateCli: MigrateCliConstructor;
export {};
