"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SqlMigrateScripter = void 0;
const core_1 = require("../core");
const migrate_scripter_1 = require("./migrate-scripter");
const util_1 = require("./util");
class SqlMigrateScripter extends migrate_scripter_1.MigrateScripter {
    constructor(comparator, builder) {
        super(comparator);
        this.builder = builder;
    }
    annotation(note) {
        this.middleCodes.push(core_1.SQL.annotation(note));
    }
    createTable(table) {
        this.middleCodes.push(this.builder.createTable(table).as(g => [
            ...table.columns.map(schema => this.tableColumnForAdd(g.column, schema)),
            g.primaryKey(table.primaryKey.name).on(table.primaryKey.columns.map(c => ({
                name: c.name,
                sort: c.isAscending ? 'ASC' : 'DESC',
            }))),
        ]));
    }
    dropTable(table) {
        this.middleCodes.push(this.builder.dropTable(table));
    }
    // useDatabase(name: string): void {
    //   this.beforeCodes.push(this.builder.use(name));
    // }
    // createDatabase(database: DatabaseSchema): void {
    //   const sql = this.builder.createDatabase(database.name);
    //   if (database.collate) {
    //     sql.collate(database.collate);
    //   }
    //   this.beforeCodes.push(sql);
    // }
    // alterDatabase(database: DatabaseSchema): void {
    //   this.beforeCodes.push(
    //     this.builder.alterDatabase(database.name).collate(database.collate!)
    //   );
    // }
    // dropDatabase(name: string): void {
    //   this.beforeCodes.push(this.builder.dropDatabase(name));
    // }
    insertSeedData(table, data) {
        const fields = table.columns
            .filter(col => !col.isCalculate)
            .map(col => col.name);
        const rows = data.map(item => {
            const row = {};
            fields.forEach(field => (row[field] = item[field]));
            return row;
        });
        const identityColumn = table.columns.find(col => col.isIdentity);
        const sql = this.builder.insert(table).values(rows);
        if (identityColumn) {
            sql.withIdentity();
        }
        this.seedDataCodes.set(table, sql);
    }
    createIndex(table, index) {
        const sql = this.builder.createIndex(index.name).on(table, index.columns.map(col => ({
            name: col.name,
            sort: col.isAscending ? 'ASC' : 'DESC',
        })));
        if (index.isUnique) {
            sql.unique();
        }
        if (index.isClustered) {
            sql.clustered();
        }
        this.middleCodes.push(sql);
    }
    dropIndex(table, name) {
        this.middleCodes.push(this.builder.dropIndex(table, name));
    }
    addForeignKey(table, fk) {
        const sql = this.builder.alterTable(table).add($ => {
            const v = $.foreignKey(fk.name)
                .on(fk.columns)
                .reference({ name: fk.referenceTable, schema: fk.referenceSchema }, fk.referenceColumns);
            if (fk.isCascade) {
                v.deleteCascade();
            }
            return v;
        });
        this.afterCodes.push(sql);
    }
    dropForeignKey(table, name) {
        this.beforeCodes.push(this.builder.alterTable(table).dropForeignKey(name));
    }
    commentColumn(table, name, comment) {
        if (comment) {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.setColumnComment(table, name, comment)));
        }
        else {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.dropColumnComment(table, name)));
        }
    }
    commentTable(table, comment) {
        if (comment) {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.setTableComment(table, comment)));
        }
        else {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.dropTableComment(table)));
        }
    }
    commentIndex(table, name, comment) {
        if (comment) {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.setIndexComment(table, name, comment)));
        }
        else {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.dropIndexComment(table, name)));
        }
    }
    commentConstraint(table, name, comment) {
        if (comment) {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.setConstraintComment(table, name, comment)));
        }
        else {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.dropConstraintComment(table, name)));
        }
    }
    commentForeignKey(table, name, comment) {
        if (comment) {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.setConstraintComment(table, name, comment)));
        }
        else {
            this.middleCodes.push(...(0, util_1.pack)(this.builder.dropConstraintComment(table, name)));
        }
    }
    setDefaultValue(table, column, defaultValue) {
        this.middleCodes.push(...(0, util_1.pack)(this.builder.setDefaultValue(table, column, core_1.SQL.raw(defaultValue))));
    }
    dropDefaultValue(table, column) {
        this.middleCodes.push(...(0, util_1.pack)(this.builder.dropDefaultValue(table, column)));
    }
    setAutoRowflag(table, column) {
        this.middleCodes.push(...(0, util_1.pack)(this.builder.setAutoRowflag(table, column)));
    }
    dropAutoRowflag(table, column) {
        this.middleCodes.push(...(0, util_1.pack)(this.builder.dropAutoRowflag(table, column)));
    }
    dropIdentity(table, column) {
        this.middleCodes.push(...(0, util_1.pack)(this.builder.dropIdentity(table, column)));
    }
    setIdentity(table, column, start, increment) {
        this.middleCodes.push(...(0, util_1.pack)(this.builder.setIdentity(table, column, start, increment)));
    }
    createSequence(sequence) {
        this.middleCodes.push(this.builder
            .createSequence(sequence)
            .as(core_1.SQL.raw(sequence.type))
            .startWith(sequence.startValue ?? 0)
            .incrementBy(sequence.increment ?? 1));
    }
    dropSequence(name) {
        this.middleCodes.push(this.builder.dropSequence(name));
    }
    addColumn(table, schema) {
        this.middleCodes.push(this.builder
            .alterTable(table)
            .add(({ column }) => this.tableColumnForAdd(column, schema)));
    }
    dropColumn(table, name) {
        this.middleCodes.push(this.builder.alterTable(table).dropColumn(name));
    }
    alterColumn(table, column) {
        this.middleCodes.push(this.builder
            .alterTable(table)
            .alterColumn(g => this.tableColumnForAlter(g, column)));
    }
    addPrimaryKey(table, pk) {
        this.middleCodes.push(this.builder.alterTable(table).add(({ primaryKey }) => primaryKey(pk.name).on(pk.columns.map(col => ({
            name: col.name,
            sort: col.isAscending ? 'ASC' : 'DESC',
        })))));
    }
    dropPrimaryKey(table, name) {
        this.middleCodes.push(this.builder.alterTable(table).dropPrimaryKey(name));
    }
    addConstraint(table, constraint) {
        this.middleCodes.push(this.builder.alterTable(table).add(g => {
            switch (constraint.kind) {
                case 'CHECK':
                    return g.check(constraint.name, core_1.SQL.raw(constraint.sql));
                // case 'UNIQUE':
                //   return g.uniqueKey(constraint.name).on(
                //     constraint.columns.map(
                //       col =>
                //         ({
                //           name: col.name,
                //           sort: col.isAscending ? 'ASC' : 'DESC',
                //         } as KeyColumn)
                //     )
                //   );
            }
        }));
    }
    dropConstraint(table, constraint) {
        switch (constraint.kind) {
            case 'CHECK':
                this.middleCodes.push(this.builder.alterTable(table).dropCheck(constraint.name));
                break;
            // case 'UNIQUE':
            //   this.middleCodes.push(
            //     this.builder.alterTable(table).dropUniqueKey(constraint.name)
            //   );
            //   break;
        }
    }
    tableColumnForAlter(g, schema) {
        const col = g(schema.name, core_1.DbType.raw(schema.type));
        if (schema.isNullable) {
            col.null();
        }
        else {
            col.notNull();
        }
        return col;
    }
    tableColumnForAdd(column, schema) {
        const col = column(schema.name, core_1.DbType.raw(schema.type));
        if (schema.isNullable) {
            col.null();
        }
        else {
            col.notNull();
        }
        if (schema.isIdentity) {
            col.identity(schema.identityStartValue, schema.identityIncrement);
        }
        if (schema.isCalculate) {
            col.as(core_1.SQL.raw(schema.calculateExpression));
        }
        if (schema.isIdentity) {
            col.identity(schema.identityStartValue, schema.identityIncrement);
        }
        if (schema.isCalculate) {
            col.as(core_1.SQL.raw(schema.calculateExpression));
        }
        if (schema.defaultValue !== null && schema.defaultValue !== undefined) {
            col.default(core_1.SQL.raw(schema.defaultValue));
        }
        return col;
    }
}
exports.SqlMigrateScripter = SqlMigrateScripter;
