"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DbContext = void 0;
const events_1 = require("events");
const queryable_1 = require("./queryable");
const repository_1 = require("./repository");
const metadata_store_1 = require("./metadata-store");
const entity_mgr_1 = require("./entity-mgr");
class DbContext {
    constructor(connection) {
        this._emitter = new events_1.EventEmitter();
        this._disposed = false;
        this._connection = connection;
        this.metadata = metadata_store_1.metadataStore.getContext(this.constructor);
    }
    get connection() {
        if (this._disposed) {
            throw new Error(`Context is disposed.`);
        }
        return this._connection;
    }
    get metadata() {
        return this._metadata;
    }
    set metadata(value) {
        this._metadata = value;
    }
    // 为节省内存，Repository 使用共享对象
    get repositories() {
        if (!this._repositories) {
            this._repositories = new Map();
        }
        return this._repositories;
    }
    // 为节省内存，Repository 使用共享对象
    get mgrs() {
        if (!this._mgrs) {
            this._mgrs = new Map();
        }
        return this._mgrs;
    }
    /**
     * 获取一个可查询对象
     */
    getRepository(Entity) {
        if (this._disposed) {
            throw new Error(`Context is disposed.`);
        }
        if (!this.repositories.get(Entity)) {
            const repo = new repository_1.Repository(this, Entity);
            repo.on('all', (event, items) => {
                this._emit(event, Entity, items);
            });
            this.repositories.set(Entity, repo);
        }
        return this.repositories.get(Entity);
    }
    /**
     * 获取一个仓储库对象
     */
    getQueryable(Entity) {
        if (this._disposed) {
            throw new Error(`Context is disposed.`);
        }
        return new queryable_1.Queryable(this, Entity);
    }
    /**
     * 内部方法，请不要使用
     * @internal
     */
    getMgr(Entity) {
        if (this._disposed) {
            throw new Error(`Context is disposed.`);
        }
        let mgr = this.mgrs.get(Entity);
        if (!mgr) {
            mgr = new entity_mgr_1.EntityMgr(Entity, this);
            this.mgrs.set(Entity, mgr);
        }
        return mgr;
    }
    get(Entity, key, options) {
        return this.getRepository(Entity).get(key, options);
    }
    insert(...args) {
        let Entity;
        let items;
        let options;
        if (args.length === 1) {
            items = args[0];
            if (Array.isArray(args[0])) {
                Entity = args[0][0].constructor;
            }
            else {
                Entity = args[0].constructor;
            }
            options = args[1];
        }
        else {
            Entity = args[0];
            items = args[1];
            options = args[2];
        }
        return this.getRepository(Entity).insert(items, options);
    }
    update(...args) {
        let Entity;
        let items;
        let options;
        if (args.length === 1) {
            items = args[0];
            if (Array.isArray(args[0])) {
                Entity = args[0][0].constructor;
            }
            else {
                Entity = args[0].constructor;
            }
            options = args[1];
        }
        else {
            Entity = args[0];
            items = args[1];
            options = args[2];
        }
        return this.getRepository(Entity).update(items, options);
    }
    delete(...args) {
        let Entity;
        let items;
        let options;
        if (args.length === 1) {
            items = args[0];
            if (Array.isArray(args[0])) {
                Entity = args[0][0].constructor;
            }
            else {
                Entity = args[0].constructor;
            }
            options = args[1];
        }
        else {
            Entity = args[0];
            items = args[1];
            options = args[2];
        }
        return this.getRepository(Entity).delete(items, options);
    }
    save(...args) {
        let Entity;
        let items;
        let options;
        if (args.length === 1) {
            items = args[0];
            if (Array.isArray(args[0])) {
                Entity = args[0][0].constructor;
            }
            else {
                Entity = args[0].constructor;
            }
            options = args[1];
        }
        else {
            Entity = args[0];
            items = args[1];
            options = args[2];
        }
        return this.getRepository(Entity).save(items, options);
    }
    on(event, handler) {
        //  this._emmiter.on(event, handler);
        return this;
    }
    _emit(event, Entity, items) {
        this._emitter.emit(event, Entity, items);
        this._emitter.emit('all', Entity, items);
    }
    off(event, handler) {
        if (!handler) {
            this._emitter.removeAllListeners(event);
        }
        else {
            this._emitter.off(event, handler);
        }
        return this;
    }
    dispose() {
        // if (this.connection.opened) {
        //   await this.connection.close();
        // }
        this._disposed = true;
    }
}
exports.DbContext = DbContext;
/**
 * 数据库上下文对象
 */
