import { Entity, EntityConstructor } from '../entity';
import { Select, Scalar } from '../../core';
import { ColumnMetadata } from './column-metadata';
import { IndexMetadata } from './index-metdata';
import { KeyMetadata } from './key-metadata';
import { RelationMetadata, SubordinateRelation, SuperiorRelation, OneToOneMetadata, OneToManyMetadata, ManyToOneMetadata, ManyToManyMetadata } from './relation-metadata';
import { DbContextConstructor } from '../db-context';
import { FetchRelations } from '../data-types';
export interface CommonEntityMetadata {
    /**
     * 是否隐式生成的
     */
    isImplicit: boolean;
    /**
     * 类型
     */
    kind: 'TABLE' | 'VIEW' | 'QUERY';
    /**
     * 表/视图名
     */
    dbName?: string;
    /**
     * 视图的SELECT语句
     */
    body?: Select;
    /**
     * 查询语句
     */
    sql?: Select;
    /**
     * 构造函数
     */
    class: EntityConstructor<Entity>;
    /**
     * 数据库上下文类
     */
    contextClass: DbContextConstructor;
    /**
     * 模型名称
     */
    className: string;
    /**
     * 主键元数据
     */
    key?: KeyMetadata;
    /**
     * 是否只读
     */
    readonly: boolean;
    /**
     * 标识列
     */
    identityColumn?: ColumnMetadata;
    /**
     * 行标记列
     */
    rowflagColumn?: ColumnMetadata;
    /**
     * 摘要描述
     */
    comment?: string;
    /**
     * 数据
     */
    data?: any[];
    readonly indexes: ReadonlyArray<IndexMetadata>;
    readonly relations: ReadonlyArray<RelationMetadata>;
    readonly columns: ReadonlyArray<ColumnMetadata>;
}
/**
 * 表格实体元数据
 */
export declare class EntityMetadata implements CommonEntityMetadata {
    schema?: string;
    /**
     * 是否隐式生成的
     */
    isImplicit: boolean;
    /**
     * 类型
     */
    kind: 'TABLE' | 'VIEW' | 'QUERY';
    /**
     * 类构造函数
     */
    class: EntityConstructor;
    /**
     * 所绑定的实体类构造函数
     */
    contextClass: DbContextConstructor;
    /**
     * 主键
     */
    key?: KeyMetadata;
    /**
     * 表/视图 名称
     */
    dbName?: string;
    /**
     * 视图主体语句
     */
    body?: Select;
    /**
     * 查询的SQL语句
     */
    sql?: Select;
    /**
     * 实体类名
     */
    className: string;
    /**
     * 是否只读
     */
    readonly: boolean;
    /**
     * 标识列
     */
    identityColumn?: ColumnMetadata<Scalar> | undefined;
    /**
     * 版本标记列
     */
    rowflagColumn?: ColumnMetadata<Scalar> | undefined;
    /**
     * 批注
     */
    comment?: string | undefined;
    /**
     * 种子数据
     */
    data?: object[] | undefined;
    private _columnMap;
    private _columns;
    private _relationMap;
    private _indexMap;
    private _reations;
    private _indexes;
    get indexes(): ReadonlyArray<IndexMetadata>;
    addIndex(index: IndexMetadata): this;
    addColumn(item: ColumnMetadata): this;
    addRelation(item: RelationMetadata): this;
    get columns(): ReadonlyArray<ColumnMetadata>;
    getColumn(name: string): ColumnMetadata | undefined;
    get relations(): ReadonlyArray<RelationMetadata>;
    getRelation(name: string): RelationMetadata | undefined;
    getIndex(name: string): IndexMetadata | undefined;
    private _detailIncludes?;
    getDetailIncludes(): FetchRelations<any> | undefined;
    getDetailRelations(): SubordinateRelation[];
    getSubordinateRelations(): SubordinateRelation[];
    getSuperiorRelations(): SuperiorRelation[];
}
export interface TableEntityMetadata extends EntityMetadata {
    kind: 'TABLE';
    /**
     * 表名
     */
    dbName: string;
    key: KeyMetadata;
    /**
     * 架构
     */
    schema?: string;
}
/**
 * 视图实体元数据
 */
export interface ViewEntityMetadata extends EntityMetadata {
    /**
     * 类型
     */
    kind: 'VIEW';
    /**
     * 表名
     */
    dbName: string;
    /**
     * 架构名称
     */
    schema?: string;
    /**
     * 查询,视图或者查询的SELECT语句
     */
    body: Select;
    /**
     * 是否只读
     */
    readonly: true;
    relations: never;
    getRelation: never;
}
/**
 * 查询实体元数据
 */
export interface QueryEntityMetadata extends EntityMetadata {
    /**
     * 是否隐式生成的
     */
    isImplicit: true;
    /**
     * 类型
     */
    kind: 'QUERY';
    /**
     * 查询,视图或者查询的SELECT语句
     */
    sql: Select;
    schema: never;
    name: never;
    /**
     * 是否只读
     */
    readonly: true;
    relations: never;
    getRelation: never;
}
/**
 * 实体成员元数据
 */
export declare type EntityMemberMetadata = ColumnMetadata | IndexMetadata | OneToOneMetadata | OneToManyMetadata | ManyToOneMetadata | ManyToManyMetadata;
