"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContextBuilder = void 0;
const metadata_1 = require("../metadata");
const metadata_store_1 = require("../metadata-store");
const util_1 = require("../metadata/util");
const entity_builder_1 = require("./entity-builder");
const property_builder_1 = require("./property-builder");
const util_2 = require("./util");
class ContextBuilder {
    constructor(ctr) {
        /**
         * 实体映射
         */
        this._entityMap = new Map();
        /**
         * 是否已经完成
         */
        this._isReady = false;
        this.metadata = new metadata_1.DbContextMetadata(ctr);
        this.metadata.className = ctr.name;
    }
    get isReady() {
        return this._isReady;
    }
    /**
     * 指定数据连接
     */
    connectWith(connectionOptions) {
        this.metadata.connection = connectionOptions;
    }
    hasComment(comment) {
        this.metadata.comment = comment;
    }
    entity(ctr, build) {
        if (this._isReady)
            throw new Error(`Context is completed, not allow this operation.`);
        if (!this._entityMap.has(ctr)) {
            const metadata = new metadata_1.EntityMetadata();
            Object.assign(metadata, {
                kind: 'TABLE',
                dbName: ctr.name,
                className: ctr.name,
                class: ctr,
                contextClass: this.metadata.class,
            });
            const eb = new entity_builder_1.EntityBuilder(this, metadata);
            this._entityMap.set(ctr, eb);
            this.metadata.addEntity(metadata);
        }
        const builder = this._entityMap.get(ctr);
        if (build) {
            build(builder);
            return this;
        }
        return builder;
    }
    // /**
    //  * 将实体绑定到表中
    //  * @param ctr
    //  */
    // table<T extends Entity>(ctr: Constructor<T>): EntityBuilder<T>;
    // table<T extends Entity>(
    //   ctr: Constructor<T>,
    //   build: (builder: EntityBuilder<T>) => void
    // ): this;
    // table<T extends Entity>(
    //   ctr: Constructor<T>,
    //   build?: (builder: EntityBuilder<T>) => void
    // ): EntityBuilder<T> | this {
    //   const builder = this.entity(ctr).asTable();
    //   if (build) {
    //     build(builder);
    //     return this;
    //   }
    //   return builder;
    // }
    // /**
    //  * 将实体绑定到视图当中
    //  * @param ctr
    //  */
    // view<T extends Entity>(ctr: Constructor<T>): ViewEntityBuilder<T>;
    // view<T extends Entity>(
    //   ctr: Constructor<T>,
    //   build: (builder: ViewEntityBuilder<T>) => void
    // ): this;
    // view<T extends Entity>(
    //   ctr: Constructor<T>,
    //   build?: (builder: ViewEntityBuilder<T>) => void
    // ): ViewEntityBuilder<T> | this {
    //   const builder = this.entity(ctr).asView();
    //   if (build) {
    //     build(builder);
    //     return this;
    //   }
    //   return builder;
    // }
    // /**
    //  * 将实体绑定到查询当中
    //  * @param ctr
    //  */
    // query<T extends Entity>(ctr: Constructor<T>): QueryEntityBuilder<T>;
    // query<T extends Entity>(
    //   ctr: Constructor<T>,
    //   build: (builder: QueryEntityBuilder<T>) => void
    // ): this;
    // query<T extends Entity>(
    //   ctr: Constructor<T>,
    //   build?: (builder: QueryEntityBuilder<T>) => void
    // ): QueryEntityBuilder<T> | this {
    //   const builder = this.entity(ctr).asQuery();
    //   if (build) {
    //     build(builder);
    //     return this;
    //   }
    //   return builder;
    // }
    /**
     * 结束建模过程并完善数据并校验完整性
     */
    ready() {
        if (this._isReady)
            throw new Error(`Context is completed, use 'context(Context, build)' will auto call the complete.`);
        if (!this.metadata.database) {
            this.metadata.database = this.metadata.className;
        }
        if (!this.metadata.globalKey) {
            this.hasGlobalKey('id', BigInt);
        }
        if (this.metadata.globalKey) {
            (0, util_2.fixColumn)(this.metadata.globalKey.column);
        }
        for (const entity of this.metadata.entities) {
            (0, util_2.fixEntity)(this, entity);
        }
        // 处理关联关系
        for (const entity of this.metadata.entities) {
            if (!(0, util_1.isTableEntity)(entity))
                continue;
            (0, util_2.fixEntityIndexes)(entity);
            (0, util_2.fixEntityRelations)(this, entity);
        }
        this._isReady = true;
        // 注册进 metadataStore中
        metadata_store_1.metadataStore.registerContext(this.metadata);
    }
    /**
     * 指定全局主键字段名称
     */
    hasGlobalKey(name, type) {
        // this.metadata.globalKeyName = name;
        // this.metadata.globalKeyType = type;
        const columnBuilder = new property_builder_1.PropertyBuilder(this, null, {
            columnName: name,
            type
        });
        this.metadata.globalKey = {
            property: name,
            column: columnBuilder.metadata
        };
        return columnBuilder;
    }
}
exports.ContextBuilder = ContextBuilder;
