import { Scalar, Select } from "../../core";
import { Entity, EntityConstructor } from "../entity";
import { EntityMetadata } from "../metadata";
import { DataTypeOf } from "../data-types";
import { ContextBuilder } from "./context-builder";
import { HasManyBuilder } from "./has-many-builder";
import { HasOneBuilder } from "./has-one-builder";
import { IndexBuilder } from "./index-builder";
import { PropertyBuilder } from "./property-builder";
import { TableKeyBuilder } from "./table-key-builder";
/**
 * 实体构造器
 * 列可以重复获取，关系不可以重复获取
 */
export declare class EntityBuilder<T extends Entity> {
    protected contextBuilder: ContextBuilder;
    readonly metadata: EntityMetadata;
    constructor(contextBuilder: ContextBuilder, metadata: EntityMetadata);
    protected readonly relationMaps: Map<string, HasOneBuilder<T, Entity> | HasManyBuilder<T, Entity>>;
    protected readonly columnMaps: Map<string, PropertyBuilder<T>>;
    private assertRelation;
    private assertKind;
    asTable(name?: string): EntityBuilder<T>;
    asTable(build: (builder: EntityBuilder<T>) => void): ContextBuilder;
    asTable(name: string, build: (builder: EntityBuilder<T>) => void): ContextBuilder;
    /**
     * 实体声明为视图
     */
    asView(body: Select<T>): this;
    asView(body: Select<T>, build: (builder: this) => void): ContextBuilder;
    asView(name: string, body: Select<T>): this;
    asView(name: string, body: Select<T>, build: (builder: this) => void): ContextBuilder;
    /**
     * 实体声明为视图
     */
    asQuery(sql: Select<T>): this;
    asQuery(sql: Select<T>, build: (builder: this) => void): ContextBuilder;
    hasName(name: string): this;
    /**
     * 声明主键
     */
    hasKey<P extends Scalar>(selector: (p: Required<T>) => P): TableKeyBuilder;
    hasKey<P extends Scalar>(constraintName: string, selector: (p: Required<T>) => P): TableKeyBuilder;
    hasIndex(name: string): IndexBuilder<T>;
    /**
     * 声明一个单一引用属性
     * @param selector
     * @param type 因typescript反射机制尚不完善，因此无法获取到属性类型，因而需要传递该属性类型参数
     * @returns
     */
    hasOne<D extends Entity>(propertySelector: (p: Required<T>) => D, type: EntityConstructor<D>): HasOneBuilder<T, D>;
    /**
     * 声明一个集体属性
     * @param selector
     * @param type 因typescript反射机制尚不完善，因此无法获取到属性类型，因而需要传递该属性类型参数
     * @returns
     */
    hasMany<D extends Entity>(selector: (p: Required<T>) => D[], type: EntityConstructor<D>): HasManyBuilder<T, D>;
    /**
     * 种子数据，必须指定所有字段值，包括Identity字段
     * 暂时只支持，数据表对象新增
     * 重复调用则追加
     */
    hasData(data: Record<string, Scalar>[]): this;
    property<P extends Scalar>(selector: (p: Required<T>) => P, type: DataTypeOf<P>): PropertyBuilder<T, P>;
    property<P extends Scalar>(selector: (p: Required<T>) => P, type: DataTypeOf<P>, build: (builder: PropertyBuilder<T>) => void): this;
    property<D extends Scalar, P extends keyof T>(property: string, type: DataTypeOf<D>): PropertyBuilder<T, D>;
    property<D extends Scalar, P extends keyof T>(property: string, type: DataTypeOf<D>, build: (builder: PropertyBuilder<T>) => void): this;
    hasComment(comment: string): Omit<this, 'hasComment'>;
    /**
     * 可将初始化代码写于此处，支持异步方法
     */
    forAsync(action: (builder: this) => Promise<void>): Promise<this>;
}
