"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityBuilder = void 0;
const util_1 = require("../util");
const has_many_builder_1 = require("./has-many-builder");
const has_one_builder_1 = require("./has-one-builder");
const index_builder_1 = require("./index-builder");
const property_builder_1 = require("./property-builder");
const table_key_builder_1 = require("./table-key-builder");
/**
 * 实体构造器
 * 列可以重复获取，关系不可以重复获取
 */
class EntityBuilder {
    constructor(contextBuilder, metadata) {
        this.contextBuilder = contextBuilder;
        this.metadata = metadata;
        this.relationMaps = new Map();
        this.columnMaps = new Map();
    }
    assertRelation(property) {
        if (this.relationMaps.has(property) || this.columnMaps.has(property)) {
            throw new Error(`Property or Relation ${property} is declared`);
        }
    }
    assertKind() {
        if (this.metadata.kind) {
            throw new Error('Not allow set a entity kind twice in modeling.');
        }
    }
    asTable(nameOrBuild, build) {
        // this.assertKind();
        let tableName;
        if (typeof nameOrBuild === 'string') {
            tableName = nameOrBuild;
        }
        else if (typeof nameOrBuild === 'function') {
            build = nameOrBuild;
        }
        const tableMetadata = this.metadata;
        this.metadata.kind = 'TABLE';
        tableMetadata.dbName = tableName || this.metadata.className;
        this.metadata.readonly = false;
        if (build) {
            build(this);
            return this.contextBuilder;
        }
        return this;
    }
    asView(nameOrBody, bodyOrBuild, build) {
        let name;
        let body;
        if (typeof nameOrBody === 'string') {
            name = nameOrBody;
            body = bodyOrBuild;
        }
        else {
            body = nameOrBody;
            build = bodyOrBuild;
        }
        this.metadata.kind = 'VIEW';
        this.metadata.dbName = name || this.metadata.className;
        this.metadata.readonly = true;
        this.metadata.body = body;
        if (build) {
            build(this);
            return this.contextBuilder;
        }
        return this;
    }
    asQuery(sql, build) {
        this.metadata.kind = 'QUERY';
        this.metadata.readonly = true;
        this.metadata.sql = sql;
        if (build) {
            build(this);
            return this.contextBuilder;
        }
        return this;
    }
    hasName(name) {
        this.metadata.dbName = name;
        return this;
    }
    hasKey(nameOrSelector, selector) {
        let constraintName;
        if (typeof nameOrSelector === 'function') {
            selector = nameOrSelector;
        }
        else {
            constraintName = nameOrSelector;
        }
        const property = (0, util_1.selectProperty)(selector);
        if (!property) {
            throw new Error('Please select a property');
        }
        if (!this.metadata.key) {
            this.metadata.key = {};
        }
        this.metadata.key = {
            property,
        };
        if (constraintName) {
            this.metadata.key.constraintName = constraintName;
        }
        // this.metadata.addIndex({
        //   name: undefined,
        //   properties: [property],
        //   columns: null,
        //   isUnique: true,
        //   isClustered
        // });
        return new table_key_builder_1.TableKeyBuilder(this, this.metadata.key);
    }
    hasIndex(name) {
        // if (!name) {
        //   name = `IX_${this.metadata.tableName || this.metadata.className}_${index}`
        // }
        let metadata = this.metadata.getIndex(name);
        if (!metadata) {
            metadata = { name };
            this.metadata.addIndex(metadata);
        }
        const builder = new index_builder_1.IndexBuilder(this.contextBuilder, this, metadata);
        return builder;
    }
    hasOne(propertyOrSelector, type) {
        const property = typeof propertyOrSelector === 'function'
            ? (0, util_1.selectProperty)(propertyOrSelector)
            : propertyOrSelector;
        if (!property) {
            throw new Error(`Entity ${this.metadata.class.name} hasOne selector mast return a property, example property name "user", 'builder.hasOne(p => p.user)'`);
        }
        this.assertRelation(property);
        const metadata = {
            property,
            referenceClass: type,
            // 追加关联关系
            referenceEntity: this.contextBuilder.entity(type).metadata,
        };
        const builder = new has_one_builder_1.HasOneBuilder(this.contextBuilder, this, metadata);
        this.relationMaps.set(property, builder);
        this.metadata.addRelation(metadata);
        return builder;
    }
    /**
     * 声明一个集体属性
     * @param selector
     * @param type 因typescript反射机制尚不完善，因此无法获取到属性类型，因而需要传递该属性类型参数
     * @returns
     */
    hasMany(selector, type) {
        const property = (0, util_1.selectProperty)(selector);
        if (!property)
            throw new Error(`Entity ${this.metadata.class.name} hasMany selector mast select a property like this: '.hasMany(p => p.listPropery)'.`);
        this.assertRelation(property);
        const metadata = {
            referenceClass: type,
            referenceEntity: this.contextBuilder.entity(type)
                .metadata,
            property: property,
            isDetail: false,
            isImplicit: false,
        };
        this.contextBuilder.entity(type);
        const builder = new has_many_builder_1.HasManyBuilder(this.contextBuilder, this, metadata);
        this.relationMaps.set(property, builder);
        this.metadata.addRelation(metadata);
        return builder;
    }
    // TODO: 待添加种子数据结构化初始
    /**
     * 种子数据，必须指定所有字段值，包括Identity字段
     * 暂时只支持，数据表对象新增
     * 重复调用则追加
     */
    // hasData(data: T[]): this
    // hasData(data: any[]): this
    hasData(data) {
        // if (this.metadata.kind !== "TABLE") {
        //   throw new Error("Seed data is allowed only on table.");
        // }
        if (data.length === 0) {
            throw new Error(`Entity ${this.metadata.class.name} hasData seed data is empty.`);
        }
        if (!this.metadata.data)
            this.metadata.data = [];
        this.metadata.data.push(...data);
        return this;
    }
    property(propertyOrselector, type, build) {
        const property = typeof propertyOrselector === 'function'
            ? (0, util_1.selectProperty)(propertyOrselector)
            : propertyOrselector;
        if (!property) {
            throw new Error(`Please select a property`);
        }
        let columnBuilder = this.columnMaps.get(property);
        if (!columnBuilder) {
            const metadata = {
                kind: 'COLUMN',
                property,
                type: type,
            };
            columnBuilder = new property_builder_1.PropertyBuilder(this.contextBuilder, this, metadata);
            this.columnMaps.set(property, columnBuilder);
            this.metadata.addColumn(metadata);
        }
        if (build) {
            build(columnBuilder);
            return this;
        }
        return columnBuilder;
    }
    hasComment(comment) {
        this.metadata.comment = comment;
        return this;
    }
    /**
     * 可将初始化代码写于此处，支持异步方法
     */
    async forAsync(action) {
        await action(this);
        return this;
    }
}
exports.EntityBuilder = EntityBuilder;
