"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.modelBuilder = exports.ModelBuilder = void 0;
const db_context_1 = require("../db-context");
const reader_1 = require("../decorators/reader");
const context_builder_1 = require("./context-builder");
class ModelBuilder {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    constructor() {
        this.contextMap = new Map();
        this.isDecoratorBuilt = false;
    }
    buildDecoratorDbContext(target) {
        const connectionOptions = (0, reader_1.getConnectionOptions)(target);
        if (connectionOptions) {
            this.context(target).metadata.connection = connectionOptions;
        }
        const contextOptions = (0, reader_1.getContextOptions)(target);
        if (contextOptions?.globalKeyName) {
            this.context(target).hasGlobalKey(contextOptions.globalKeyName, contextOptions.globalKeyType);
        }
        if (contextOptions) {
            Object.assign(this.context(target).metadata, contextOptions);
        }
        const comment = (0, reader_1.getComment)(target);
        if (comment) {
            this.context(target).hasComment(comment);
        }
    }
    buildDecoratorEntity(target) {
        const entityOptions = (0, reader_1.getEntityOptions)(target);
        const { contextGetter, ...assignable } = entityOptions;
        const entityBuilder = this.context(contextGetter?.() || db_context_1.DbContext).entity(target);
        // Object.assign(entityBuilder.metadata, assignable);
        if (assignable.dbName) {
            entityBuilder.hasName(assignable.dbName);
        }
        // if (assignable.readonly) {
        //   entityBuilder.isReadonly()
        // }
        const columnProperties = (0, reader_1.getEntityColumns)(target);
        if (columnProperties) {
            for (const property of columnProperties) {
                const columnOptions = (0, reader_1.getColumnOptions)(target, property);
                if (columnOptions) {
                    const columnBuilder = entityBuilder.property(p => p[property], columnOptions.type);
                    if (columnOptions.columnName) {
                        columnBuilder.hasColumnName(columnOptions.columnName);
                    }
                    if (columnOptions.dbType) {
                        columnBuilder.hasType(columnOptions.dbType);
                    }
                    if (columnOptions.defaultValueGetter) {
                        columnBuilder.hasDefaultValue(columnOptions.defaultValueGetter());
                    }
                    if (columnOptions.isNullable !== undefined) {
                        if (columnOptions.isNullable) {
                            columnBuilder.isNullable();
                        }
                        else {
                            columnBuilder.isRequired();
                        }
                    }
                    if (columnOptions.generator) {
                        columnBuilder.isAutogen(columnOptions.generator);
                    }
                    if (columnOptions.isIdentity) {
                        columnBuilder.isIdentity(columnOptions.identityStartValue, columnOptions.identityIncrement);
                    }
                    if (columnOptions.isCalculate) {
                        columnBuilder.isCalculated(columnOptions.calculateExpressionGetter());
                    }
                    if (columnOptions.isRowflag) {
                        columnBuilder.isRowflag();
                    }
                    const comment = (0, reader_1.getComment)(target, property);
                    if (comment) {
                        columnBuilder.hasComment(comment);
                    }
                }
                const indexOptions = (0, reader_1.getIndexOptions)(target, property);
                if (indexOptions) {
                    entityOptions.indexes.push(indexOptions);
                }
            }
        }
        if (assignable.kind === 'TABLE') {
            entityBuilder.asTable();
            const keyOptions = (0, reader_1.getEntityKeyOptions)(target);
            if (keyOptions) {
                const keyBuilder = entityBuilder.hasKey(p => p[keyOptions.property]);
                Object.assign(keyBuilder.metadata, keyOptions);
            }
            const relationProperties = (0, reader_1.getEntityRelations)(target);
            if (relationProperties) {
                for (const property of relationProperties) {
                    const relationOptions = (0, reader_1.getRelationOptions)(target, property);
                    let builder;
                    switch (relationOptions.kind) {
                        case 'ONE_TO_ONE': {
                            const map = entityBuilder
                                .hasOne(p => p[property], relationOptions.referenceEntityGetter())
                                .withOne(relationOptions.referenceProperty
                                ? p => p[relationOptions.referenceProperty]
                                : undefined);
                            if (relationOptions.isPrimary) {
                                builder = map.isPrimary();
                                builder.isDetail(relationOptions.isDetail);
                            }
                            else {
                                let bd;
                                builder = bd = map.hasForeignKey(relationOptions.foreignProperty
                                    ? p => p[relationOptions.foreignProperty]
                                    : undefined);
                                if (relationOptions.isRequired) {
                                    bd.isRequired();
                                }
                            }
                            const comment = (0, reader_1.getComment)(target, property);
                            if (comment) {
                                builder.hasComment(comment);
                            }
                            break;
                        }
                        case 'ONE_TO_MANY': {
                            builder = entityBuilder
                                .hasMany(p => p[property], relationOptions.referenceEntityGetter())
                                .withOne(relationOptions.referenceProperty
                                ? (p) => p[relationOptions.referenceProperty]
                                : undefined);
                            builder.isDetail(relationOptions.isDetail);
                            break;
                        }
                        case 'MANY_TO_ONE': {
                            builder = entityBuilder
                                .hasOne(p => p[property], relationOptions.referenceEntityGetter())
                                .withMany(relationOptions.referenceProperty
                                ? p => p[relationOptions.referenceProperty]
                                : undefined);
                            if (relationOptions.foreignProperty) {
                                builder.hasForeignKey(p => p[relationOptions.foreignProperty]);
                            }
                            if (relationOptions.isRequired) {
                                builder.isRequired();
                            }
                            const comment = (0, reader_1.getComment)(target, property);
                            if (comment) {
                                builder.hasComment(comment);
                            }
                            break;
                        }
                        case 'MANY_TO_MANY': {
                            builder = entityBuilder
                                .hasMany(p => p[property], relationOptions.referenceEntityGetter())
                                .withMany(relationOptions.referenceProperty
                                ? (p) => p[relationOptions.referenceProperty]
                                : undefined);
                            builder.isDetail(relationOptions.isDetail);
                            // 查找中间表声明
                            for (const relationEntity of (0, reader_1.getDecorateEntities)()) {
                                const among = (0, reader_1.getAmong)(relationEntity);
                                if (!among)
                                    continue;
                                const leftMatch = among.leftEntityGetter() === target &&
                                    among.rightEngityGetter() ===
                                        relationOptions.referenceEntityGetter();
                                const rightMatch = among.rightEngityGetter() === target &&
                                    among.leftEntityGetter() ===
                                        relationOptions.referenceEntityGetter();
                                if (leftMatch && rightMatch) {
                                    // 声明表
                                    builder.hasRelationTable(relationEntity, relationOptions.relationProperty
                                        ? p => p[relationOptions.relationProperty]
                                        : undefined);
                                    if (relationOptions.relationProperty) {
                                        if (leftMatch && among.leftProperty) {
                                            entityBuilder
                                                .hasMany(p => p[relationOptions.relationProperty], relationEntity)
                                                .withOne((p) => p[among.leftProperty]);
                                        }
                                        else if (rightMatch && among.rightProperty) {
                                            entityBuilder
                                                .hasMany(p => p[relationOptions.relationProperty], relationEntity)
                                                .withOne((p) => p[among.rightProperty]);
                                        }
                                    }
                                }
                            }
                            break;
                        }
                    }
                }
            }
            entityOptions.indexes.forEach(indexOptions => {
                const indexBuilder = entityBuilder.hasIndex(indexOptions.name);
                indexBuilder.withProperties(Array.isArray(indexOptions.properties)
                    ? p => indexOptions.properties
                    : indexOptions.properties);
                if (indexOptions.isUnique !== undefined) {
                    indexBuilder.isUnique(indexOptions.isUnique);
                }
            });
            const among = (0, reader_1.getAmong)(target);
            if (among) {
                if (among.leftProperty && !entityBuilder.metadata.getRelation(among.leftProperty)) {
                    entityBuilder
                        .hasOne(p => p[among.leftProperty], among.leftEntityGetter())
                        .withMany();
                }
                if (among.rightProperty && !entityBuilder.metadata.getRelation(among.rightProperty)) {
                    entityBuilder.hasOne(p => p[among.rightProperty], among.rightEngityGetter());
                }
            }
            if (entityOptions.seedDatas) {
                entityBuilder.hasData(entityOptions.seedDatas);
            }
        }
        else if (assignable.kind === 'VIEW') {
            if (!assignable.body) {
                throw new Error(`View entity must specify body statement.`);
            }
            entityBuilder.asView(assignable.body);
        }
        else if (assignable.kind === 'QUERY') {
            if (!assignable.sql) {
                throw new Error(`Query entity must specify body statement.`);
            }
            entityBuilder.asQuery(assignable.sql);
        }
    }
    buildDecorators() {
        // 在使用context之前初始化指令模型
        if (!this.isDecoratorBuilt) {
            // 必须提前，避免死死循环
            this.isDecoratorBuilt = true;
            const decorateContexts = (0, reader_1.getDecorateContexts)();
            if (decorateContexts) {
                for (const context of decorateContexts) {
                    this.buildDecoratorDbContext(context);
                }
            }
            const decoratorEntities = (0, reader_1.getDecorateEntities)();
            if (decoratorEntities) {
                for (const entity of decoratorEntities) {
                    this.buildDecoratorEntity(entity);
                }
            }
        }
    }
    context(context, build) {
        this.buildDecorators();
        let builder = this.contextMap.get(context);
        if (!builder) {
            builder = new context_builder_1.ContextBuilder(context);
            this.contextMap.set(context, builder);
        }
        if (build) {
            build(builder);
            return this;
        }
        return builder;
    }
    /**
     * 完成建模工作后，必须调用该方法，才能正常使用ORM功能。
     * 此方法请在ORM准备完毕时调用
     */
    ready() {
        this.buildDecorators();
        for (const ctxBuilder of this.contextMap.values()) {
            if (!ctxBuilder.isReady) {
                ctxBuilder.ready();
            }
        }
    }
}
exports.ModelBuilder = ModelBuilder;
ModelBuilder.instance = new ModelBuilder();
exports.modelBuilder = ModelBuilder.instance;
