"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Queryable = void 0;
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
/* eslint-disable @typescript-eslint/no-inferrable-types */
/* eslint-disable @typescript-eslint/no-explicit-any */
const core_1 = require("../core");
const util_1 = require("./util");
const metadata_store_1 = require("./metadata-store");
const util_2 = require("./metadata/util");
const sql_1 = require("../core/sql");
const { select } = core_1.SQL;
const ROWSET_ALIAS = '__t__';
// eslint-disable-next-line @typescript-eslint/ban-types
class Queryable {
    constructor(context, Ctr, rowset) {
        this.context = context;
        this._withDetail = false;
        if (!Ctr && !rowset) {
            throw new Error(`Ctr and rowset One of them must be specified.`);
        }
        if (Ctr) {
            this._metadata = metadata_store_1.metadataStore.getEntity(Ctr);
            if (!rowset) {
                this._rowset = (0, util_2.makeRowset)(Ctr);
            }
        }
        if (rowset) {
            this._rowset = rowset;
        }
        if ((0, util_2.isTableEntity)(this._metadata)) {
            this._tableMetadata = this._metadata;
        }
    }
    get mgr() {
        if (!this._metadata) {
            throw new Error(`Mgr is only allowed for entities in queryable.`);
        }
        return this.context.getMgr(this._metadata.class);
    }
    get sql() {
        if (!this._sql) {
            this._sql = select(this._rowset.star).from(this._rowset);
        }
        return this._sql;
    }
    get executor() {
        return this.context.connection;
    }
    /**
     * 异步迭代器，延迟查询的关键点
     */
    [Symbol.asyncIterator]() {
        let result;
        let i = 0;
        return {
            next: async () => {
                if (!result) {
                    result = await this.fetchAll();
                }
                if (i < result.length) {
                    return { value: result[i++], done: false };
                }
                return { value: undefined, done: true };
            },
        };
    }
    skip(count) {
        if (this.sql.$offset !== undefined) {
            throw new Error(`Skip cannot be used twice, If you want to use a subquery, call .fork() first.`);
        }
        this.sql.offset(count);
        return this;
    }
    take(count) {
        if (this.sql.$limit !== undefined) {
            throw new Error(`Take cannot be used twice, If you want to use a subquery, call .fork() first.`);
        }
        this.sql.limit(count);
        return this;
    }
    // /**
    //  * 克隆当前对象用于添加信息，以免污染当前对象
    //  */
    fork(rowset) {
        if (this._metadata) {
            rowset = (0, util_2.aroundRowset)(rowset, this._metadata);
        }
        const queryable = new Queryable(this.context, this._metadata?.class, rowset);
        queryable._withDetail = this._withDetail;
        queryable._includes = this._includes;
        queryable._metadata = this._metadata;
        return queryable;
    }
    /**
     * 过滤数据并返回一个新的Queryable
     */
    filter(condition) {
        this.sql.andWhere(condition(this._rowset));
        return this;
    }
    count() {
        // this._sql.$columns = [
        //   SQL.count(this._rowset.$field(this._metadata!.key!.property! as any)).as('count')
        // ];
        return this.map(() => ({
            count: core_1.SQL.count(1),
        }));
    }
    sort(sorts) {
        if (this.sql.$sorts) {
            throw new Error(`Cannot be used .sort(...) twice.`);
        }
        this.sql.orderBy(sorts(this._rowset));
        return this;
        // return this.fork(
        //   select(this.rowset.star)
        //     .from(this.rowset)
        //     .orderBy(sorts(this.rowset))
        //     .as(ROWSET_ALIAS)
        // );
    }
    /**
     * 添加过滤条件，并限定返回头一条记录
     */
    find(filter) {
        return this.filter(filter).take(1);
    }
    union(...queries) {
        const sql = this.sql;
        queries.forEach(query => {
            sql.unionAll(query.sql);
        });
        return this;
    }
    /**
     * 返回一个新的类型
     */
    map(results) {
        const rowset = this._sql ? this._sql.as(ROWSET_ALIAS) : this._rowset;
        const sql = select(results(rowset)).from(rowset);
        return new Queryable(this.context, undefined, sql.as(ROWSET_ALIAS));
    }
    groupBy(results, groups, having) {
        const rowset = this._sql ? this.sql.as(ROWSET_ALIAS) : this._rowset;
        const sql = select(results(rowset)).groupBy(...groups(this._rowset));
        if (having) {
            sql.having(having(rowset));
        }
        return new Queryable(this.context, undefined, sql.as(ROWSET_ALIAS));
    }
    join(entity, on, results) {
        const rightRowset = (0, util_2.makeRowset)(entity);
        const leftRowset = this._sql ? this._sql.as(ROWSET_ALIAS) : this._rowset;
        const newRowset = core_1.SQL.select(results(leftRowset, rightRowset))
            .from(this._rowset)
            .join(rightRowset, on(leftRowset, rightRowset), true)
            .as(ROWSET_ALIAS);
        return new Queryable(this.context, undefined, newRowset);
    }
    /**
     * 查询关联导航属性
     */
    include(props) {
        if (!this._tableMetadata) {
            throw new Error('Queryable.include is allow use in table entity only.');
        }
        // const queryable = this.fork(this.rowset);
        // queryable._includes = props;
        // return queryable;
        this._includes = (0, util_1.mergeFetchRelations)(this._includes, props);
        return this;
    }
    /**
     * 同时返回明细属性
     * 即被标记为明细属性的导航属性
     */
    withDetail() {
        if (!this._tableMetadata) {
            throw new Error('Queryable.withDetail is allow use in table entity only.');
        }
        this._withDetail = true;
        return this;
    }
    getSqlForExecute() {
        if (this._sql) {
            return this._sql;
        }
        // 减少SELECT嵌套
        if (sql_1.NamedSelect.isNamedSelect(this._rowset)) {
            return this._rowset.$select;
        }
        else {
            return core_1.SQL.select(this._rowset.star).from(this._rowset);
        }
    }
    async loadRelations(items) {
        let includes;
        if (this._withDetail) {
            includes = this._tableMetadata.getDetailIncludes();
        }
        if (this._includes) {
            includes = (0, util_1.mergeFetchRelations)(includes, this._includes);
        }
        if (includes) {
            await this.context
                .getMgr(this._metadata.class)
                .loadRelations(Array.isArray(items) ? items : [items], includes);
        }
    }
    /**
     * 执行查询并将结果转换为数组，并获取所有数据返回一个数组
     */
    async fetchAll() {
        const { rows } = await this.executor.query(this.getSqlForExecute());
        if (!this._metadata) {
            return rows;
        }
        const items = rows.map(row => this.mgr.toEntity(row));
        if (this._tableMetadata) {
            await this.loadRelations(items);
        }
        return items;
    }
    /**
     * 执行查询，并获取第一行记录
     */
    async fetchFirst() {
        const { rows } = await this.executor.query(this.getSqlForExecute().limit(1));
        if (rows.length <= 0) {
            throw new Error(`No result exists on this query.`);
        }
        if (!this._metadata) {
            return rows[0];
        }
        const item = this.mgr.toEntity(rows[0]);
        if (this._tableMetadata) {
            await this.loadRelations(item);
        }
        return item;
    }
}
exports.Queryable = Queryable;
