import { XTable, DefaultRowObject, Executor } from '../core';
import { Queryable } from './queryable';
import { TableEntityMetadata } from './metadata';
import { DbContext } from './db-context';
import { Entity, EntityConstructor, EntityInstance, EntityKeyType } from './entity';
import { DbEvents, FetchRelations, RelationKeyOf, RepositoryEventHandler } from './data-types';
export declare type FetchOptions<T> = {
    includes?: FetchRelations<T>;
    /**
     * 是否连同明细属性一并查询
     * 默认为false
     * 如果为true，则系统会自动查询明细relation属性，并且不需要指定`includes`
     */
    withDetail?: boolean;
};
export declare type SaveOptions<T = any> = {
    /**
     * 保存时不保存关系属性，仅保存自身
     */
    withoutRelations?: boolean | RelationKeyOf<T>[];
};
/**
 * 删除选项
 */
export declare type DeleteOptions = {
    /**
     * 连同明细项一并删除，无论是否声明
     */
    withDetail?: boolean;
};
export declare class Repository<T extends Entity> {
    protected readonly context: DbContext;
    ctr: EntityConstructor<T>;
    protected metadata: TableEntityMetadata;
    protected rowset: XTable<DefaultRowObject>;
    private _emitter;
    constructor(context: DbContext, ctr: EntityConstructor<T>);
    protected get connection(): Executor;
    private get mgr();
    /**
     * 创建一个可查询对象
     */
    query(): Queryable<T>;
    /**
     * 通过主键查询一个项，
     * 默认不使用缓存
     */
    get(key: EntityKeyType, options?: FetchOptions<T>): Promise<EntityInstance<T>>;
    insert(items: T | T[], options?: SaveOptions<T>): Promise<void>;
    private _insert;
    update(items: T | T[], options?: SaveOptions<T>): Promise<void>;
    /**
     * 修改一个数据项
     * 本方法会对全字段更新，无论是否修改与否。
     * @param item
     */
    protected _update(items: T | T[], options?: SaveOptions<any>): Promise<void>;
    private _delete;
    /**
     * 删除指定数据项
     */
    delete(items: EntityKeyType | T | T[], options?: DeleteOptions): Promise<void>;
    private _deleteDetail;
    /**
     * 已存在的则修改，未存在的则新增
     * 如果存在ROWFLAG字段，则在提交之前会进行核对
     * 保存完后，进行保存的数据会被更新为最新数据
     */
    save(items: T | T[], options?: SaveOptions<T>): Promise<void>;
    /**
     * 数据提交事件
     */
    on(event: 'save', handler: RepositoryEventHandler<T>): this;
    on(event: 'insert', handler: RepositoryEventHandler<T>): this;
    on(event: 'update', handler: RepositoryEventHandler<T>): this;
    on(event: 'delete', handler: RepositoryEventHandler<T>): this;
    on(event: 'saved', handler: RepositoryEventHandler<T>): this;
    on(event: 'inserted', handler: RepositoryEventHandler<T>): this;
    on(event: 'updated', handler: RepositoryEventHandler<T>): this;
    on(event: 'deleted', handler: RepositoryEventHandler<T>): this;
    on(event: 'all', handler: (event: DbEvents, items: T[]) => void): this;
    private _emit;
    off(event: 'save', handler?: RepositoryEventHandler<T>): this;
    off(event: 'insert', handler?: RepositoryEventHandler<T>): this;
    off(event: 'update', handler?: RepositoryEventHandler<T>): this;
    off(event: 'delete', handler?: RepositoryEventHandler<T>): this;
    off(event: 'saved', handler: RepositoryEventHandler<T>): this;
    off(event: 'inserted', handler: RepositoryEventHandler<T>): this;
    off(event: 'updated', handler: RepositoryEventHandler<T>): this;
    off(event: 'deleted', handler: RepositoryEventHandler<T>): this;
    off(event: 'all', handler: (event: DbEvents, items: T[]) => void): this;
    private _save;
    existsKey(key: EntityKeyType): Promise<boolean>;
    private _setProperty;
    /**
     * 保存引用的依赖项
     * @param item
     */
    protected saveSuperiors(item: T, withoutRelations?: RelationKeyOf<any>[]): Promise<void>;
    /**
     * 保存外键关系
     */
    protected saveSubordinates(item: T, withoutRelations?: RelationKeyOf<any>[], skipCompare?: boolean): Promise<void>;
    private _savePrimaryOneToOneRelation;
    private _saveOneToManyRelation;
    private _saveManyToManyRelation;
}
