"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Repository = void 0;
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/ban-types */
const core_1 = require("../core");
const queryable_1 = require("./queryable");
const async_event_1 = require("./async-event");
const util_1 = require("./metadata/util");
const metadata_store_1 = require("./metadata-store");
const util_2 = require("./util");
class Repository {
    constructor(context, ctr) {
        this.context = context;
        this.ctr = ctr;
        this._emitter = new async_event_1.AsyncEventEmitter();
        const metadata = metadata_store_1.metadataStore.getEntity(ctr);
        if (!(0, util_1.isTableEntity)(metadata)) {
            throw new Error(`Entitymgr is only allowed for tableentities.`);
        }
        this.metadata = metadata;
        // this.metadata = metadataStore.getEntity(
        //   Entity as EntityConstructor<Entity>
        // ) as TableEntityMetadata;
        if (this.metadata.readonly) {
            throw new Error(`Repository must instance of table entity`);
        }
        if (!this.metadata) {
            throw new Error(`Only allow register entity constructor.`);
        }
        this.rowset = (0, util_1.makeRowset)(ctr);
    }
    get connection() {
        return this.context.connection;
    }
    get mgr() {
        return this.context.getMgr(this.metadata.class);
    }
    /**
     * 创建一个可查询对象
     */
    query() {
        return new queryable_1.Queryable(this.context, this.metadata.class);
    }
    /**
     * 通过主键查询一个项，
     * 默认不使用缓存
     */
    async get(key, options) {
        let includes;
        if (options?.withDetail) {
            includes = this.metadata.getDetailIncludes();
        }
        includes = (0, util_2.mergeFetchRelations)(options?.includes, includes);
        return await this.mgr.fetchByKey(key, {
            includes,
        });
    }
    async insert(items, options) {
        await this._insert(items, options);
    }
    async _insert(items, options) {
        if (!Array.isArray(items)) {
            items = [items];
        }
        this._emit('insert', items, this.context);
        for (const item of items) {
            if (options?.withoutRelations !== true) {
                await this.saveSuperiors(item, options?.withoutRelations === false
                    ? undefined
                    : options?.withoutRelations);
            }
            const row = await this.mgr.getInsertValues(this.rowset, item);
            await this.connection.insert(this.rowset, row);
            const key = this.metadata.key.column.isIdentity
                ? core_1.SQL.identityValue(this.metadata.dbName, this.metadata.key.column.columnName)
                : Reflect.get(item, this.metadata.key.property);
            const added = (await this.connection.find(this.rowset, this.rowset.$field(this.metadata.key.property).eq(key)));
            this.mgr.toEntity(added, item);
            if (options?.withoutRelations !== true) {
                await this.saveSubordinates(item, options?.withoutRelations === false
                    ? undefined
                    : options?.withoutRelations, true);
            }
        }
        this._emit('inserted', items, this.context);
    }
    async update(items, options) {
        await this._update(items, options);
    }
    /**
     * 修改一个数据项
     * 本方法会对全字段更新，无论是否修改与否。
     * @param item
     */
    async _update(items, options) {
        if (!Array.isArray(items)) {
            items = [items];
        }
        await this._emit('update', items, this.context);
        for (const item of items) {
            if (options?.withoutRelations !== true) {
                // 保存父表项
                await this.saveSuperiors(item, options?.withoutRelations === false
                    ? undefined
                    : options?.withoutRelations);
            }
            const changes = await this.mgr.getUpdateChanges(this.rowset, item);
            const where = await this.mgr.getUpdateWhere(this.rowset, item);
            const lines = await this.connection.update(this.rowset, changes, where);
            if (lines === 0) {
                throw new Error(`The data to be modified does not exist or has been modified or deleted.`);
            }
            const updated = await this.connection.find(this.rowset, this.mgr.getQueryWhere(this.rowset, item));
            this.mgr.toEntity(updated, item);
            if (options?.withoutRelations !== true) {
                // 保存子项
                await this.saveSubordinates(item, options?.withoutRelations === false
                    ? undefined
                    : options?.withoutRelations);
            }
        }
        await this._emit('updated', items, this.context);
    }
    async _delete(data, options) {
        let items;
        if ((0, core_1.isBaseScalar)(data)) {
            const item = await this.get(data);
            if (!item) {
                throw new Error(`Entity ${this.metadata.className} key ${data} not found for delete.`);
            }
            items = [item];
        }
        else {
            if (!Array.isArray(data)) {
                items = [data];
            }
            else {
                items = data;
            }
        }
        if (items.length === 0) {
            throw new Error('Items must have more than or equals one of record.');
        }
        this._emit('delete', items, this.context);
        for (const item of items) {
            if (options?.withDetail) {
                await this._deleteDetail(item);
            }
            const where = this.mgr.getQueryWhere(this.rowset, item);
            const lines = await this.connection.delete(this.rowset, where);
            if (lines !== 1) {
                throw new Error(`The data to be delete does not exist or has been changed or deleted. ${item}`);
            }
        }
        this._emit('deleted', items, this.context);
    }
    /**
     * 删除指定数据项
     */
    async delete(items, options) {
        return this._delete(items, options);
    }
    async _deleteDetail(item) {
        for (const relation of this.metadata.relations) {
            if ((0, util_1.isPrimaryOneToOne)(relation)) {
                if (!relation.isDetail)
                    continue;
                const detail = (await this.mgr.fetchRelation([item], relation))[0];
                if (detail) {
                    await this.context
                        .getRepository(relation.referenceClass)
                        .delete(detail, { withDetail: true });
                }
            }
            else if ((0, util_1.isOneToMany)(relation)) {
                if (!relation.isDetail)
                    continue;
                const details = (await this.mgr.fetchRelation([item], relation))[0];
                if (details?.length > 0) {
                    await this.context
                        .getRepository(relation.referenceClass)
                        .delete(details, { withDetail: true });
                }
            }
            else if ((0, util_1.isManyToMany)(relation)) {
                // 多对多关系仅中间表
                const middles = (await this.mgr.fetchRelation([item], relation.relationRelation))[0];
                if (middles?.length > 0) {
                    await this.context
                        .getRepository(relation.relationRelation.referenceClass)
                        .delete(middles, { withDetail: true });
                }
            }
        }
    }
    /**
     * 已存在的则修改，未存在的则新增
     * 如果存在ROWFLAG字段，则在提交之前会进行核对
     * 保存完后，进行保存的数据会被更新为最新数据
     */
    async save(items, options) {
        if (!Array.isArray(items)) {
            items = [items];
        }
        await this._emit('save', items, this.context);
        await this._save(items);
        await this._emit('saved', items, this.context);
    }
    on(event, handler) {
        this._emitter.on(event, handler);
        return this;
    }
    async _emit(event, items, context) {
        await this._emitter.emit(event, items);
        await this._emitter.emit('all', items);
    }
    off(event, handler) {
        this._emitter.off(event, handler);
        return this;
    }
    async _save(items, options) {
        if (!Array.isArray(items)) {
            items = [items];
        }
        for (const item of items) {
            const keyValue = Reflect.get(item, this.metadata.key.property);
            if (this.metadata.key.column.isIdentity) {
                if (keyValue !== undefined) {
                    // 否则为修改数据
                    await this._update(item, options);
                }
                else {
                    // 如果存在主键，则表示更新数据
                    await this._insert(item, options);
                }
            }
            else {
                // 自动生成列的主键可能为空
                if (keyValue !== undefined && (await this.existsKey(keyValue))) {
                    // 否则为修改数据
                    await this._update(item, options);
                }
                else {
                    // 如果存在主键，则表示更新数据
                    await this._insert(item, options);
                }
            }
        }
    }
    // 判断是否存在主键值
    async existsKey(key) {
        const result = await this.query()
            .filter(p => p[this.metadata.key.property].eq(key))
            .count()
            .fetchFirst();
        return (result?.count ?? 0) > 0;
    }
    _setProperty(item, column, value) {
        // 更新引用外键
        if (Reflect.has(item, column.property)) {
            Reflect.set(item, column.property, value);
        }
        else {
            Reflect.defineProperty(item, column.property, {
                enumerable: !column.isImplicit,
                value,
            });
        }
    }
    /**
     * 保存引用的依赖项
     * @param item
     */
    async saveSuperiors(item, withoutRelations) {
        for (const relation of this.metadata.relations) {
            if (withoutRelations?.includes?.(relation.property))
                continue;
            if (Reflect.get(item, relation.property) === undefined)
                continue;
            if ((0, util_1.isForeignOneToOne)(relation) || (0, util_1.isManyToOne)(relation)) {
                const dependent = Reflect.get(item, relation.property);
                if (!dependent) {
                    // 切断关联关系
                    this._setProperty(item, relation.foreignColumn, null);
                    continue;
                }
                await this.context
                    .getRepository(relation.referenceClass)
                    ._save(dependent, {
                    withoutRelations: [relation.referenceRelation.property],
                });
                const foreignKey = Reflect.get(dependent, relation.referenceEntity.key.property);
                this._setProperty(item, relation.foreignColumn, foreignKey);
            }
        }
    }
    /**
     * 保存外键关系
     */
    async saveSubordinates(item, withoutRelations, skipCompare = false) {
        for (const relation of this.metadata.relations) {
            if (withoutRelations?.includes?.(relation.property))
                continue;
            if (Reflect.get(item, relation.property) === undefined)
                continue;
            if ((0, util_1.isOneToMany)(relation)) {
                await this._saveOneToManyRelation(item, relation, skipCompare);
            }
            else if ((0, util_1.isManyToMany)(relation)) {
                await this._saveManyToManyRelation(item, relation, skipCompare);
            }
            else if ((0, util_1.isPrimaryOneToOne)(relation)) {
                await this._savePrimaryOneToOneRelation(item, relation);
            }
        }
    }
    async _savePrimaryOneToOneRelation(item, relation) {
        const subItem = Reflect.get(item, relation.property);
        const itemKey = Reflect.get(item, this.metadata.key.property);
        const repo = this.context.getRepository(relation.referenceClass);
        repo._setProperty(subItem, relation.referenceRelation.foreignColumn, itemKey);
        await repo._save(subItem, {
            withoutRelations: [relation.referenceRelation.property],
        });
    }
    async _saveOneToManyRelation(item, relation, skipCompare = false) {
        const subItems = Reflect.get(item, relation.property) || [];
        const itemKey = Reflect.get(item, this.metadata.key.property);
        const repo = this.context.getRepository(relation.referenceClass);
        // 设置主键
        for (const subItem of subItems) {
            repo._setProperty(subItem, relation.referenceRelation.foreignColumn, itemKey);
        }
        if (skipCompare) {
            return await repo._save(subItems, {
                withoutRelations: [relation.referenceRelation.property],
            });
        }
        const subSnapshots = (await this.context
            .getMgr(this.metadata.class)
            .fetchRelation([item], relation))[0];
        const itemsMap = {};
        const snapshotMap = {};
        subItems.forEach((subItem) => {
            itemsMap[Reflect.get(subItem, relation.referenceEntity.key.property)] =
                subItem;
        });
        subSnapshots.forEach((subItem) => {
            snapshotMap[Reflect.get(subItem, relation.referenceEntity.key.property)] =
                subItem;
        });
        // 保存(新增或者修改)子项,跳过父属性
        await repo._save(subItems, {
            withoutRelations: [relation.referenceRelation.property],
        });
        // 删除多余的子项
        for (const subItem of subSnapshots) {
            const subKey = Reflect.get(subItem, relation.referenceEntity.key.property);
            if (!itemsMap[subKey]) {
                await repo.delete(subItem);
            }
        }
    }
    async _saveManyToManyRelation(item, relation, skipCompare = false) {
        // 多对多关系仅会删除中间关系表，并不会删除引用表
        // 获取一对多关系中间表快照
        // 先存储引用表
        const subRepo = this.context.getRepository(relation.referenceClass);
        const relationRepo = this.context.getRepository(relation.relationRelation.referenceClass);
        const itemKey = Reflect.get(item, this.metadata.key.property);
        const subItems = Reflect.get(item, relation.property);
        const makeRelationItem = (subItem) => {
            const subKey = Reflect.get(subItem, relation.referenceEntity.key.property);
            const relationItem = new relation.relationClass();
            relationRepo._setProperty(relationItem, relation.relationRelation.referenceRelation.foreignColumn, itemKey);
            relationRepo._setProperty(relationItem, relation.referenceRelation.relationRelation.referenceRelation
                .foreignColumn, subKey);
            return relationItem;
        };
        if (skipCompare) {
            await subRepo._save(subItems, {
                withoutRelations: [relation.referenceRelation.property],
            });
            const relationItems = subItems.map((subItem) => makeRelationItem(subItem));
            await relationRepo._insert(relationItems, {
                withoutRelations: [relation.relationRelation.property],
            });
            return;
        }
        // 取中间表快照
        const relationSnapshots = (await this.context
            .getMgr(this.metadata.class)
            .fetchRelation([item], relation.relationRelation))[0];
        await subRepo._save(subItems, {
            withoutRelations: [relation.referenceRelation.property],
        });
        const snapshotMap = {};
        relationSnapshots.forEach((subItem) => {
            const subKey = subItem[relation.referenceRelation.relationRelation.referenceRelation
                .foreignProperty];
            snapshotMap[subKey] = subItem;
        });
        const itemsMap = {};
        subItems.forEach((subItem) => {
            const subKey = subItem[relation.referenceEntity.key.property];
            itemsMap[subKey] = subItem;
        });
        const newSubItems = subItems.filter((subItem) => {
            const subKey = Reflect.get(subItem, relation.referenceEntity.key.property);
            return !snapshotMap[subKey];
        });
        const newRelationItems = newSubItems.map((newSubItem) => makeRelationItem(newSubItem));
        // 新增未关联的关系
        await relationRepo._insert(newRelationItems);
        const removedRelationItems = relationSnapshots.filter(relationItem => {
            const subKey = Reflect.get(relationItem, relation.referenceRelation.relationRelation.referenceRelation
                .foreignProperty);
            return !itemsMap[subKey];
        });
        // 删除多余的关联关
        await relationRepo._delete(removedRelationItems);
    }
}
exports.Repository = Repository;
